@@ -50,7 +50,7 @@ termed `pywbem loggers`:
   responses between the pywbem client and WBEM server. This logs the HTTP
   request data and response data including HTTP headers and CIM-XML payload.
 
-Pywbem uses the :attr:`py:logging.DEBUG` logging level for both loggers.
+Pywbem uses the ``DEBUG`` logging level (see :ref:`py:levels`) for both loggers.
 
 Pywbem adds a null handler to the logger named `'pywbem'`, in order to prevent
 the "No handlers could be found for logger ..." warning.
@@ -58,11 +58,11 @@ This follows best practices recommended in `Configuring logging for a library
 <https://docs.python.org/2/howto/logging.html#configuring-logging-for-a-library>`_
 and in several articles, for example in `this article
 <http://pieces.openpolitics.com/2012/04/python-logging-best-practices/>`_.
-Because this warning is no longer issued on Python 3.4 and higher, pywbem
-adds a null handler only on Python 2.6 and 2.7.
+Because this warning is no longer issued on Python 3.5 and higher, pywbem
+adds a null handler only on Python 2.7.
 
-Because pywbem logs only at the :attr:`py:logging.DEBUG` logging level, these
-log events will not be printed by the Python root logger by default, and
+Because pywbem logs only at the ``DEBUG`` logging level (see :ref:`py:levels`),
+these log events will not be printed by the Python root logger by default, and
 therefore it is not necessary that pywbem attaches a null handler to any of its
 loggers.
 
@@ -231,7 +231,7 @@ from ._utils import _format
 __all__ = ['configure_logger', 'configure_loggers_from_string',
            'LOGGER_API_CALLS_NAME', 'LOGGER_HTTP_NAME', 'LOGGER_SIMPLE_NAMES',
            'LOG_DESTINATIONS', 'DEFAULT_LOG_DESTINATION', 'LOG_DETAIL_LEVELS',
-           'DEFAULT_LOG_DETAIL_LEVEL']
+           'DEFAULT_LOG_DETAIL_LEVEL', 'DEFAULT_LOG_FILENAME']
 
 _CONN = None
 
@@ -257,8 +257,9 @@ LOGGER_SIMPLE_NAMES = ['api', 'http', 'all']
 #:
 #: * `'file'` - Log to a file (requires filename to be specified). The file
 #:   logger appends to the logger file defined by filename.
-#: * `'stderr'` - Log to the standard error stream of the Python process
-LOG_DESTINATIONS = ['file', 'stderr']
+#: * `'stderr'` - Log to the standard error stream of the Python process.
+#: * `'off'` - Disable logging.
+LOG_DESTINATIONS = ['file', 'stderr', 'off']
 
 #: Default log destination if not supplied to the logging configuration
 #: functions.
@@ -314,7 +315,9 @@ def configure_logger(simple_name, log_dest=None,
         :data:`~pywbem._logging.LOG_DESTINATIONS` and the Python logging
         parameters of the loggers will be configured accordingly for their
         log handler, message format, and with a logging level of
-        :attr:`py:logging.DEBUG`.
+        ``DEBUG`` (see :ref:`py:levels`).
+
+        The value `'off'` disables logging for the affected pywbem loggers.
 
         If `None`, the Python logging parameters of the loggers will not be
         changed.
@@ -369,6 +372,7 @@ def configure_logger(simple_name, log_dest=None,
 
     global _CONN  # pylint: disable=global-statement
     if _CONN is None:
+        # pylint: disable=import-outside-toplevel
         from . import WBEMConnection
         _CONN = WBEMConnection
 
@@ -460,6 +464,8 @@ def configure_loggers_from_string(log_configuration_str,
         'api=stderr,http=file'  # Set 'pywbem.api' logger to stderr output and
                                 # 'pywbem.http' logger to file output, both
                                 # with default detail level.
+
+        'all=off'               # Disables logging for both pywbem loggers.
     """  # noqa: E501
     # pylint: enable=line-too-long
 
