@@ -22,6 +22,7 @@ Operation recorder interface and implementations.
 """
 
 from __future__ import print_function, absolute_import
+import io
 from collections import namedtuple
 try:
     from collections import OrderedDict
@@ -32,7 +33,7 @@ import logging
 
 import six
 import yaml
-from yaml.representer import RepresenterError
+import yamlloader
 
 from ._nocasedict import NocaseDict
 from ._cim_obj import CIMInstance, CIMInstanceName, CIMClass, CIMClassName, \
@@ -43,9 +44,6 @@ from ._exceptions import CIMError
 from ._logging import LOGGER_API_CALLS_NAME, LOGGER_HTTP_NAME
 from ._utils import _ensure_unicode, _format
 
-if six.PY2:
-    import codecs  # pylint: disable=wrong-import-order
-
 
 __all__ = ['BaseOperationRecorder', 'TestClientRecorder',
            'LogOperationRecorder',
@@ -61,88 +59,6 @@ else:
 OpArgsTuple = namedtuple("OpArgsTuple", ["method", "args"])
 
 
-def _represent_ordereddict(dump, tag, mapping, flow_style=None):
-    """PyYAML representer function for OrderedDict.
-    This is needed for yaml.safe_dump() to support OrderedDict.
-
-    Courtesy:
-    https://blog.elsdoerfer.name/2012/07/26/make-pyyaml-output-an-ordereddict/
-    """
-    value = []
-    node = yaml.MappingNode(tag, value, flow_style=flow_style)
-    if dump.alias_key is not None:
-        dump.represented_objects[dump.alias_key] = node
-    best_style = True
-    if hasattr(mapping, 'items'):
-        mapping = mapping.items()
-    for item_key, item_value in mapping:
-        node_key = dump.represent_data(item_key)
-        node_value = dump.represent_data(item_value)
-        if not (isinstance(node_key, yaml.ScalarNode) and
-                not node_key.style):
-            best_style = False    # pylint: disable=bad-indentation
-        if not (isinstance(node_value, yaml.ScalarNode) and
-                not node_value.style):
-            best_style = False    # pylint: disable=bad-indentation
-        value.append((node_key, node_value))
-    if flow_style is None:
-        if dump.default_flow_style is not None:
-            node.flow_style = dump.default_flow_style
-        else:
-            node.flow_style = best_style
-    return node
-
-
-yaml.SafeDumper.add_representer(
-    OrderedDict,
-    lambda dumper, value:
-    _represent_ordereddict(dumper, u'tag:yaml.org,2002:map', value))
-
-
-# Tag for CIMDateTime serialization in yaml files
-CIMDATETIME_TAG = '!CIMDateTime'
-
-
-def _cimdatetime_representer(dumper, cimdatetime):
-    """
-    PyYAML representer function for CIMDateTime objects.
-    This is needed for yaml.safe_dump() to support CIMDateTime.
-    """
-    cimdatetime_str = str(cimdatetime)
-    node = dumper.represent_scalar(CIMDATETIME_TAG, cimdatetime_str)
-    return node
-
-
-yaml.SafeDumper.add_representer(CIMDateTime, _cimdatetime_representer)
-
-
-def _cimdatetime_constructor(loader, node):
-    """
-    PyYAML constructor function for CIMDateTime objects.
-    This is needed for yaml.safe_load() to support CIMDateTime.
-    """
-    cimdatetime_str = loader.construct_scalar(node)
-    cimdatetime = CIMDateTime(cimdatetime_str)
-    return cimdatetime
-
-
-yaml.SafeLoader.add_constructor(CIMDATETIME_TAG, _cimdatetime_constructor)
-
-
-# Some monkey-patching for better diagnostics:
-def _represent_undefined(self, data):
-    """Raises flag for objects that cannot be represented"""
-    raise RepresenterError(
-        _format("Cannot represent an object: {0!A} of type: {1}; "
-                "yaml_representers: {2!A}, "
-                "yaml_multi_representers: {3!A}",
-                data, type(data), self.yaml_representers.keys(),
-                self.yaml_multi_representers.keys()))
-
-
-yaml.SafeDumper.represent_undefined = _represent_undefined
-
-
 class OpArgs(OpArgsTuple):
     """
     A named tuple representing the name and input arguments of the invocation
@@ -323,6 +239,14 @@ class BaseOperationRecorder(object):
         self._conn_id = None
         self.reset()
 
+    def copy(self):
+        """
+        *New in pywbem 1.3.*
+
+        Abstract: Return a deep copy of the object with internal state reset.
+        """
+        raise NotImplementedError  # Must be implemented by subclasses
+
     def enable(self):
         """
         Enable the recorder.
@@ -384,11 +308,8 @@ class BaseOperationRecorder(object):
 
             recorder_file.close()
         """
-        if six.PY2:
-            # Open with codecs to define text mode
-            return codecs.open(filename, mode=file_mode, encoding='utf-8')
-
-        return open(filename, file_mode, encoding='utf8')
+        # pylint: disable=consider-using-with
+        return io.open(filename, file_mode, encoding='utf-8')
 
     def reset(self, pull_op=None):
         """Reset all the attributes in the class. This also allows setting
@@ -593,14 +514,31 @@ class LogOperationRecorder(BaseOperationRecorder):
             self.apilogger = logging.getLogger(LOGGER_API_CALLS_NAME)
             self.httplogger = logging.getLogger(LOGGER_HTTP_NAME)
 
+    def copy(self):
+        """
+        *New in pywbem 1.3.*
+
+        Return a deep copy of the object with internal state reset.
+
+        The user-specifiable attributes of the object are deep-copied, and all
+        other internal state (e.g. staged operations) is reset.
+        """
+        cpy = LogOperationRecorder(
+            conn_id=self._conn_id,
+            detail_levels=self.detail_levels,
+        )  # init makes copies of mutable parameters
+        return cpy
+
     def set_detail_level(self, detail_levels):
         """
         Sets the detail levels from the input dictionary in detail_levels.
+
+        The input dictionary is copied.
         """
         if detail_levels is None:
             return
 
-        self.detail_levels = detail_levels
+        self.detail_levels = dict(detail_levels)  # dict is mutable, so we copy
         if 'api' in detail_levels:
             self.api_detail_level = detail_levels['api']
         if 'http' in detail_levels:
@@ -656,15 +594,6 @@ class LogOperationRecorder(BaseOperationRecorder):
         if self.enabled and self.api_detail_level is not None and \
                 self.apilogger.isEnabledFor(logging.DEBUG):
 
-            # TODO: future bypassed code to only ouput name and method if the
-            # detail is summary.  We are not doing this because this is
-            # effectively the same information in the response so the only
-            # additional infomation is the time stamp.
-
-            # if self.api_detail_level == summary:
-            #    self.apilogger.debug('Request:%s %s', self._conn_id, method)
-            #    return
-
             # Order kwargs.  Note that this is done automatically starting
             # with python 3.6
             kwstr = ', '.join([('{0}={1!r}'.format(key, kwargs[key]))
@@ -689,8 +618,8 @@ class LogOperationRecorder(BaseOperationRecorder):
             # Format the 'summary' and 'paths' detail_levels
             if self.api_detail_level == 'summary':  # pylint: disable=R1705
                 if isinstance(ret, list):
-                    if ret:
-                        ret_type = type(ret[0]).__name__ if ret else ""
+                    if len(ret) > 0:
+                        ret_type = type(ret[0]).__name__
                         return _format("list of {0}; count={1}",
                                        ret_type, len(ret))
                     return "Empty"
@@ -739,11 +668,10 @@ class LogOperationRecorder(BaseOperationRecorder):
 
             else:    # format result
                 # test if type is tuple (subclass of tuple but not type tuple)
-                # pylint: disable=unidiomatic-typecheck
                 qrc = ""
                 # format open/pull response
-                if isinstance(ret, tuple) and \
-                        type(ret) is not tuple:  # pylint: disable=C0123
+                # pylint: disable=unidiomatic-typecheck
+                if isinstance(ret, tuple) and type(ret) is not tuple:
                     try:    # test if field instances or paths
                         rtn_data = ret.instances
                         data_str = 'instances'
@@ -903,22 +831,39 @@ class TestClientRecorder(BaseOperationRecorder):
             An open file that each test case will be written to.  This file
             should have been opened in text mode.
 
-            Since there are differences between python 2 and 3 in opening
+            Since there are differences between Python 2 and 3 in opening
             files in text mode, the static method
             :meth:`~pywbem.BaseOperationRecorder.open_file`
-            can be used to open the file or python 2/3 compatible open::
+            can be used to open the file.
 
-              from io import open
-                  f = open('blah.log', encoding='utf-8')
+            Alternatively, the Python 2/3 compatible :func:`py:io.open` can be
+            used.
 
-        Example::
+        Examples::
 
           recorder = TestClientRecorder(
               BaseOperationRecorder.open_file('recorder.log'))
+
+          recorder = TestClientRecorder(
+              io.open('recorder.log', mode='w', encoding='utf-8')
         """
         super(TestClientRecorder, self).__init__()
         self._fp = fp
 
+    def copy(self):
+        """
+        *New in pywbem 1.3.*
+
+        Return a copy of the object with internal state reset and reusing the
+        file-like object.
+
+        The attribute for the `fp` init parameter is reused by the returned
+        object without copying it, and any other internal state (e.g. staged
+        operations) is reset.
+        """
+        cpy = TestClientRecorder(fp=self._fp)
+        return cpy
+
     def record(self, pywbem_args, pywbem_result, http_request, http_response):
         """
         Function that records the invocation of a single
@@ -1008,9 +953,13 @@ class TestClientRecorder(BaseOperationRecorder):
         testcases = []
         testcases.append(testcase)
 
-        # The file is open in text mode, so we produce a unicode string
-        data = yaml.safe_dump(testcases, encoding=None, allow_unicode=True,
-                              default_flow_style=False, indent=4)
+        # The file is open in text mode, so we produce a unicode string by
+        # setting encoding=None and allow_unicode=True.
+        data = yaml.dump(
+            testcases, encoding=None, allow_unicode=True,
+            default_flow_style=False, indent=4,
+            Dumper=yamlloader.ordereddict.CSafeDumper)
+
         data = data.replace('\n\n', '\n')  # YAML dump duplicates newlines
         self._fp.write(data)
         self._fp.flush()
@@ -1022,34 +971,44 @@ class TestClientRecorder(BaseOperationRecorder):
         test_client yaml format.
         """
 
-        # namedtuple is subclass of tuple so it is instance of tuple but
-        # not type tuple. Cvt to dictionary and cvt dict to yaml.
-        # pylint: disable=unidiomatic-typecheck, no-else-return
+        # pylint: disable=unidiomatic-typecheck
         if isinstance(obj, tuple) and type(obj) is not tuple:
+            # Handle namedtuple objects.
+            # Because collections.namedtuple() is a factory function and there
+            # is no common superclass for the generated namedtuple types other
+            # than tuple itself, the detection of this case is somewhat
+            # creative: The namedtuple object is an instance of tuple but does
+            # not have type tuple.
+            # In order to record the tuple item names, namedtuple objects are
+            # converted to a dictionary first.
             ret_dict = obj._asdict()
             return self.toyaml(ret_dict)
         if isinstance(obj, (list, tuple)):
+            # Note that namedtuple objects are handeled above.
             ret = []
-            # This does not handle namedtuple
             for item in obj:
                 ret.append(self.toyaml(item))
             return ret
-        elif isinstance(obj, (dict, NocaseDict)):
+        if isinstance(obj, (dict, NocaseDict)):
             ret_dict = OrderedDict()
             for key in obj.keys():  # get keys in original case
                 ret_dict[key] = self.toyaml(obj[key])
             return ret_dict
-        elif obj is None:
+        if obj is None:
             return obj
-        elif isinstance(obj, six.binary_type):
+        if isinstance(obj, six.binary_type):
             return obj.decode("utf-8")
-        elif isinstance(obj, six.text_type):
+        if isinstance(obj, six.text_type):
             return obj
-        elif isinstance(obj, CIMInt):
+        if isinstance(obj, bool):
+            # The check for bool must be before any integer checks, because
+            # bool is a subclass of int in Python.
+            return obj
+        if isinstance(obj, CIMInt):
             # CIMInt is _Longint and therefore may exceed the value range of
             # int in Python 2. Therefore, we convert it to _Longint.
             return _Longint(obj)
-        elif isinstance(obj, six.integer_types):
+        if isinstance(obj, six.integer_types):
             # This case must be after CIMInt, because CIMInt is _Longint and
             # would match a long value in Python 2.
             # We don't convert six.integer_types to _Longint, because the value
@@ -1058,31 +1017,31 @@ class TestClientRecorder(BaseOperationRecorder):
             # and long do not inherit from each other, so it is likely best if
             # we just don't convert.
             return obj
-        elif isinstance(obj, bool):
-            return obj
-        elif isinstance(obj, CIMFloat):
+        if isinstance(obj, CIMFloat):
             return float(obj)
-        elif isinstance(obj, CIMDateTime):
+        if isinstance(obj, CIMDateTime):
             return str(obj)
-        elif isinstance(obj, datetime):
+        if isinstance(obj, datetime):
             return CIMDateTime(obj)
-        elif isinstance(obj, timedelta):
+        if isinstance(obj, timedelta):
             return CIMDateTime(obj)
-        elif isinstance(obj, CIMInstance):
+        if isinstance(obj, CIMInstance):
             ret_dict = OrderedDict()
             ret_dict['pywbem_object'] = 'CIMInstance'
             ret_dict['classname'] = self.toyaml(obj.classname)
             ret_dict['properties'] = self.toyaml(obj.properties)
+            ret_dict['qualifiers'] = self.toyaml(obj.qualifiers)
             ret_dict['path'] = self.toyaml(obj.path)
             return ret_dict
-        elif isinstance(obj, CIMInstanceName):
+        if isinstance(obj, CIMInstanceName):
             ret_dict = OrderedDict()
             ret_dict['pywbem_object'] = 'CIMInstanceName'
             ret_dict['classname'] = self.toyaml(obj.classname)
             ret_dict['namespace'] = self.toyaml(obj.namespace)
+            ret_dict['host'] = self.toyaml(obj.host)
             ret_dict['keybindings'] = self.toyaml(obj.keybindings)
             return ret_dict
-        elif isinstance(obj, CIMClass):
+        if isinstance(obj, CIMClass):
             ret_dict = OrderedDict()
             ret_dict['pywbem_object'] = 'CIMClass'
             ret_dict['classname'] = self.toyaml(obj.classname)
@@ -1090,15 +1049,16 @@ class TestClientRecorder(BaseOperationRecorder):
             ret_dict['properties'] = self.toyaml(obj.properties)
             ret_dict['methods'] = self.toyaml(obj.methods)
             ret_dict['qualifiers'] = self.toyaml(obj.qualifiers)
+            ret_dict['path'] = self.toyaml(obj.path)
             return ret_dict
-        elif isinstance(obj, CIMClassName):
+        if isinstance(obj, CIMClassName):
             ret_dict = OrderedDict()
             ret_dict['pywbem_object'] = 'CIMClassName'
             ret_dict['classname'] = self.toyaml(obj.classname)
             ret_dict['host'] = self.toyaml(obj.host)
             ret_dict['namespace'] = self.toyaml(obj.namespace)
             return ret_dict
-        elif isinstance(obj, CIMProperty):
+        if isinstance(obj, CIMProperty):
             ret_dict = OrderedDict()
             ret_dict['pywbem_object'] = 'CIMProperty'
             ret_dict['name'] = self.toyaml(obj.name)
@@ -1112,7 +1072,7 @@ class TestClientRecorder(BaseOperationRecorder):
             ret_dict['propagated'] = self.toyaml(obj.propagated)
             ret_dict['qualifiers'] = self.toyaml(obj.qualifiers)
             return ret_dict
-        elif isinstance(obj, CIMMethod):
+        if isinstance(obj, CIMMethod):
             ret_dict = OrderedDict()
             ret_dict['pywbem_object'] = 'CIMMethod'
             ret_dict['name'] = self.toyaml(obj.name)
@@ -1122,17 +1082,18 @@ class TestClientRecorder(BaseOperationRecorder):
             ret_dict['parameters'] = self.toyaml(obj.parameters)
             ret_dict['qualifiers'] = self.toyaml(obj.qualifiers)
             return ret_dict
-        elif isinstance(obj, CIMParameter):
+        if isinstance(obj, CIMParameter):
             ret_dict = OrderedDict()
             ret_dict['pywbem_object'] = 'CIMParameter'
             ret_dict['name'] = self.toyaml(obj.name)
             ret_dict['type'] = self.toyaml(obj.type)
             ret_dict['reference_class'] = self.toyaml(obj.reference_class)
+            ret_dict['embedded_object'] = self.toyaml(obj.embedded_object)
             ret_dict['is_array'] = self.toyaml(obj.is_array)
             ret_dict['array_size'] = self.toyaml(obj.array_size)
             ret_dict['qualifiers'] = self.toyaml(obj.qualifiers)
             return ret_dict
-        elif isinstance(obj, CIMQualifier):
+        if isinstance(obj, CIMQualifier):
             ret_dict = OrderedDict()
             ret_dict['pywbem_object'] = 'CIMQualifier'
             ret_dict['name'] = self.toyaml(obj.name)
@@ -1144,7 +1105,7 @@ class TestClientRecorder(BaseOperationRecorder):
             ret_dict['overridable'] = self.toyaml(obj.overridable)
             ret_dict['translatable'] = self.toyaml(obj.translatable)
             return ret_dict
-        elif isinstance(obj, CIMQualifierDeclaration):
+        if isinstance(obj, CIMQualifierDeclaration):
             ret_dict = OrderedDict()
             ret_dict['pywbem_object'] = 'CIMQualifierDeclaration'
             ret_dict['name'] = self.toyaml(obj.name)
@@ -1158,7 +1119,7 @@ class TestClientRecorder(BaseOperationRecorder):
             ret_dict['overridable'] = self.toyaml(obj.overridable)
             ret_dict['translatable'] = self.toyaml(obj.translatable)
             return ret_dict
-        else:
-            raise TypeError(
-                _format("Invalid type in TestClientRecorder.toyaml(): {0} {1}",
-                        obj.__class__.__name__, type(obj)))
+
+        raise TypeError(
+            _format("Invalid type in TestClientRecorder.toyaml(): {0} {1}",
+                    obj.__class__.__name__, type(obj)))
