@@ -17,17 +17,20 @@
 """
 *New in pywbem 0.9 as experimental and finalized in 0.10.*
 
-The WBEM server API encapsulates certain functionality of a WBEM server for use
-by a WBEM client application, such as determining the Interop namespace of the
-server, or the management profiles advertised by the server.
+The WBEM server library API of pywbem encapsulates selected functionality of a
+WBEM server for use by a WBEM client application, such as determining the
+Interop namespace and other basic information about the server, or the
+management profiles advertised by the server.
 
 This chapter has the following sections:
 
-* :ref:`Example <Server Example>` - An example on how to use the API.
+* :ref:`Example <Server Example>` - An example on how to use the WBEM server
+  library API.
 
 * :ref:`WBEMServer` - The :class:`~pywbem.WBEMServer` class serves as a general
   access point for clients to WBEM servers. It allows determining the Interop
-  namespace of the server, or the advertised management profiles.
+  namespace of the server and other basic information about the server, or the
+  advertised management profiles.
 
 .. _`Server Example`:
 
@@ -137,7 +140,12 @@ class WBEMServer(object):
         'interop',
         'root/interop',
         'root/PG_Interop',
-        # TODO: Clarify which OpenPegasus versions need root/PGInterOp?
+        # OpenPegasus before version 2.12.0 defined only PG_Interop as the
+        # interop namespace. Using other namespaces was a manual
+        # modification of at least the pegasus/mak/configschema.mak file.
+        # Starting in version 2.12.0 a configuration variable was defined
+        # so that OpenPegasus could be build with any of the specified names
+        # for interop namespace (PEGASUS_INTEROP_NAMESPACE)
     ]
 
     #: A class variable with the possible names of CIM classes for
@@ -145,6 +153,7 @@ class WBEMServer(object):
     #: namespaces on the WBEM server.
     NAMESPACE_CLASSNAMES = [
         'CIM_Namespace',
+        'CIM_WBEMServerNamespace',
         '__Namespace',
     ]
 
@@ -238,11 +247,7 @@ class WBEMServer(object):
         Raises:
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
-            ModelError: An issue with the model implemented by the WBEM server.
-            CIMError: CIM_ERR_NOT_FOUND, Interop namespace could not be
-              determined.
-            CIMError: CIM_ERR_NOT_FOUND, Namespace class could not be
-              determined.
+            ModelError: An error with the model implemented by the WBEM server.
         """
         if self._namespace_classname is None:
             self._determine_namespaces()
@@ -251,17 +256,13 @@ class WBEMServer(object):
     @property
     def namespaces(self):
         """
-        :class:`py:list` of :term:`string`: Names of all namespaces of the
+        list of :term:`string`: Names of all namespaces of the
         WBEM server.
 
         Raises:
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
-            ModelError: An issue with the model implemented by the WBEM server.
-            CIMError: CIM_ERR_NOT_FOUND, Interop namespace could not be
-              determined.
-            CIMError: CIM_ERR_NOT_FOUND, Namespace class could not be
-              determined.
+            ModelError: An error with the model implemented by the WBEM server.
         """
         if self._namespaces is None:
             self._determine_namespaces()
@@ -270,7 +271,7 @@ class WBEMServer(object):
     @property
     def namespace_paths(self):
         """
-        :class:`py:list` of :class:`~pywbem.CIMInstanceName`: Instance paths
+        list of :class:`~pywbem.CIMInstanceName`: Instance paths
         of the CIM instances in the Interop namespace that represent the
         namespaces of the WBEM server.
 
@@ -283,11 +284,7 @@ class WBEMServer(object):
         Raises:
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
-            ModelError: An issue with the model implemented by the WBEM server.
-            CIMError: CIM_ERR_NOT_FOUND, Interop namespace could not be
-              determined.
-            CIMError: CIM_ERR_NOT_FOUND, Namespace class could not be
-              determined.
+            ModelError: An error with the model implemented by the WBEM server.
         """
         if self._namespace_paths is None:
             self._determine_namespaces()
@@ -371,7 +368,7 @@ class WBEMServer(object):
     @property
     def profiles(self):
         """
-        :class:`py:list` of :class:`~pywbem.CIMInstance`: The
+        list of :class:`~pywbem.CIMInstance`: The
         `CIM_RegisteredProfile` instances representing all management profiles
         advertised by the WBEM server.
 
@@ -385,12 +382,17 @@ class WBEMServer(object):
             self._determine_profiles()
         return self._profiles
 
-    def create_namespace(self, namespace):
+    def create_namespace(self, namespace, verbose=False):
         """
         Create the specified CIM namespace in the WBEM server and
         update this WBEMServer object to reflect the new namespace
         there.
 
+        This method cannot create an Interop namespace because creating the
+        an Interop namespace with client operations depends on the prior
+        existence of an Interop namespace, and creating additional Interop
+        namespaces if one already exists is usually prevented by servers.
+
         This method attempts the following approaches for creating the
         namespace, in order, until an approach succeeds:
 
@@ -402,8 +404,11 @@ class WBEMServer(object):
            widely implemented yet.
 
         2. Issuing the `CreateInstance` operation using the CIM class
-           representing namespaces ('PG_Namespace' for OpenPegasus,
-           and 'CIM_Namespace' otherwise), against the Interop namespace.
+           representing namespaces in the WBEM server, against the Interop
+           namespace.
+
+           An exception is OpenPegasus, where 'PG_Namespace' is used instead
+           of 'CIM_Namespace'.
 
            This approach is typically supported in WBEM servers that
            support the creation of CIM namespaces. This approach is
@@ -419,6 +424,9 @@ class WBEMServer(object):
               The namespace may contain leading and a trailing slash, both of
               which will be ignored.
 
+            verbose (:class:`py:bool`):
+              Verbose mode: Print a message about the namespace creation.
+
         Returns:
 
           :term:`unicode string`: The specified CIM namespace name in its
@@ -427,20 +435,19 @@ class WBEMServer(object):
         Raises:
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
-            ModelError: An issue with the model implemented by the WBEM server.
-            CIMError: CIM_ERR_ALREADY_EXISTS, Specified namespace already
-              exists in the WBEM server.
-            CIMError: CIM_ERR_NOT_FOUND, Interop namespace could not be
-              determined.
-            CIMError: CIM_ERR_NOT_FOUND, Unexpected number of
-              `CIM_ObjectManager` instances.
-            CIMError: CIM_ERR_FAILED, Unexpected number of
-              central instances of WBEM Server profile.
+            ModelError: An error with the model implemented by the WBEM server.
         """
 
         std_namespace = _ensure_unicode(namespace.strip('/'))
 
-        ws_profiles = self.get_selected_profiles('DMTF', 'WBEM Server')
+        if verbose:
+            print("Creating namespace {} (in WBEMServer)".
+                  format(std_namespace))
+
+        try:
+            ws_profiles = self.get_selected_profiles('DMTF', 'WBEM Server')
+        except (CIMError, ModelError):
+            ws_profiles = None
         if ws_profiles:
 
             # Use approach 1: Method defined in WBEM Server profile
@@ -450,8 +457,7 @@ class WBEMServer(object):
             ws_profile_inst = ws_profiles_sorted[-1]  # latest version
             ws_insts = self.get_central_instances(ws_profile_inst.path)
             if len(ws_insts) != 1:
-                raise CIMError(
-                    CIM_ERR_FAILED,
+                raise ModelError(
                     _format("Unexpected number of central instances of WBEM "
                             "Server profile: {0!A}",
                             [i.path for i in ws_insts]))
@@ -477,8 +483,7 @@ class WBEMServer(object):
                     raise
             else:
                 if ret_val != 0:
-                    raise CIMError(
-                        CIM_ERR_FAILED,
+                    raise ModelError(
                         _format("The CreateWBEMServerNamespace() method is "
                                 "implemented but failed: {0}",
                                 out_params['Errors']))
@@ -487,13 +492,15 @@ class WBEMServer(object):
 
             # Use approach 2: CreateInstance of CIM class for namespaces
 
+            # For the new namespace, we use the same class name that is used for
+            # already existing namespaces in this particular WBEM server.
+            ns_classname = self.namespace_classname  # Determines if needed
+
             # For OpenPegasus, use 'PG_Namespace' class to account for issue
             # when using 'CIM_Namespace'. See OpenPegasus bug 10112:
             # https://bugzilla.openpegasus.org/show_bug.cgi?id=10112
-            if self.brand == "OpenPegasus":
+            if self.brand == "OpenPegasus" and ns_classname == 'CIM_Namespace':
                 ns_classname = 'PG_Namespace'
-            else:
-                ns_classname = 'CIM_Namespace'
 
             ns_inst = CIMInstance(ns_classname)
 
@@ -519,7 +526,8 @@ class WBEMServer(object):
             ns_inst['SystemCreationClassName'] = \
                 self.cimom_inst['SystemCreationClassName']
 
-            self.conn.CreateInstance(ns_inst, namespace=self.interop_ns)
+            interop_ns = self.interop_ns  # Determines the Interop namespace
+            self.conn.CreateInstance(ns_inst, namespace=interop_ns)
 
         # Refresh the list of namespaces in this object to include the one
         # we just created.
@@ -530,7 +538,7 @@ class WBEMServer(object):
 
         return std_namespace
 
-    def delete_namespace(self, namespace):
+    def delete_namespace(self, namespace, verbose=False):
         """
         Delete the specified CIM namespace in the WBEM server and
         update this WBEMServer object to reflect the removed namespace
@@ -539,6 +547,9 @@ class WBEMServer(object):
         The specified namespace must be empty (i.e. must not contain any
         classes, instances, or qualifier types.
 
+        This method cannot delete the Interop namespace because servers will
+        usually prevent their deletion.
+
         This method attempts the following approaches for deleting the
         namespace, in order, until an approach succeeds:
 
@@ -565,6 +576,9 @@ class WBEMServer(object):
               The namespace may contain leading and a trailing slash, both of
               which will be ignored.
 
+            verbose (:class:`py:bool`):
+              Verbose mode: Print a message about the namespace creation.
+
         Returns:
 
           :term:`unicode string`: The specified CIM namespace name in its
@@ -573,15 +587,18 @@ class WBEMServer(object):
         Raises:
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
-            ModelError: An issue with the model implemented by the WBEM server.
+            ModelError: An error with the model implemented by the WBEM server.
             CIMError: CIM_ERR_NOT_FOUND, Specified namespace does not exist.
             CIMError: CIM_ERR_NAMESPACE_NOT_EMPTY, Specified namespace is not
               empty.
-            Additional CIM errors.
         """
 
         std_namespace = _ensure_unicode(namespace.strip('/'))
 
+        if verbose:
+            print("Deleting namespace {} (in WBEMServer)".
+                  format(std_namespace))
+
         # Use approach 1: DeleteInstance of CIM class for namespaces
 
         # Refresh the list of namespaces in this object to make sure
@@ -591,7 +608,8 @@ class WBEMServer(object):
         if std_namespace not in self.namespaces:
             raise CIMError(
                 CIM_ERR_NOT_FOUND,
-                _format("Specified namespace does not exist: {0!A}",
+                _format("pywbem detected that the specified namespace does not "
+                        "exist: {0!A}",
                         std_namespace),
                 conn_id=self.conn.conn_id)
 
@@ -612,7 +630,8 @@ class WBEMServer(object):
         if class_paths or quals:
             raise CIMError(
                 CIM_ERR_NAMESPACE_NOT_EMPTY,
-                _format("Specified namespace {0!A} is not empty; it contains "
+                _format("pywbem detected that the specified namespace {0!A} "
+                        "is not empty; it contains "
                         "{1} top-level classes and {2} qualifier types",
                         std_namespace, len(class_paths), len(quals)),
                 conn_id=self.conn.conn_id)
@@ -656,7 +675,7 @@ class WBEMServer(object):
 
         Returns:
 
-          :class:`py:list` of :class:`~pywbem.CIMInstance`: The
+          list of :class:`~pywbem.CIMInstance`: The
           `CIM_RegisteredProfile` instances representing the filtered
           subset of the management profiles advertised by the WBEM server.
 
@@ -665,11 +684,12 @@ class WBEMServer(object):
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
             CIMError: CIM_ERR_NOT_FOUND, Interop namespace could not be
               determined.
-            KeyError: If an instance in the list of profiles is incomplete
+            ModelError: If an instance in the list of profiles is incomplete
               and does not include the required properties.
         """
 
-        org_vm = ValueMapping.for_property(self, self.interop_ns,
+        interop_ns = self.interop_ns  # Determines the Interop namespace
+        org_vm = ValueMapping.for_property(self, interop_ns,
                                            'CIM_RegisteredProfile',
                                            'RegisteredOrganization')
         org_lower = registered_org.lower() \
@@ -684,28 +704,28 @@ class WBEMServer(object):
             try:
                 inst_org_value = inst['RegisteredOrganization']
             except KeyError:
-                raise KeyError(
+                raise ModelError(
                     _format("CIM_RegisteredProfile instance in namespace "
                             "{0!A} does not have a property "
                             "'RegisteredOrganization'",
-                            self.interop_ns))
+                            interop_ns))
             inst_org = org_vm.tovalues(inst_org_value)
             try:
                 inst_name = inst['RegisteredName']
             except KeyError:
-                raise KeyError(
+                raise ModelError(
                     _format("CIM_RegisteredProfile instance in namespace "
                             "{0!A} does not have a property "
                             "'RegisteredName'",
-                            self.interop_ns))
+                            interop_ns))
             try:
                 inst_version = inst['RegisteredVersion']
             except KeyError:
-                raise KeyError(
+                raise ModelError(
                     _format("CIM_RegisteredProfile instance in namespace "
                             "{0!A} does not have a property "
                             "'RegisteredVersion'",
-                            self.interop_ns))
+                            interop_ns))
 
             inst_org_lower = inst_org.lower() \
                 if inst_org is not None else None
@@ -850,14 +870,14 @@ class WBEMServer(object):
 
         Returns:
 
-          :class:`py:list` of :class:`~pywbem.CIMInstanceName`: The instance
+          list of :class:`~pywbem.CIMInstanceName`: The instance
           paths of the central instances of the implementation of the
           management profile.
 
         Raises:
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
-            ModelError: Model implementation errors on the server.
+            ModelError: An error with the model implemented by the WBEM server.
             ValueError: User errors regarding input parameter values.
             TypeError: User errors regarding input parameter types.
         """
@@ -1074,8 +1094,7 @@ class WBEMServer(object):
         Raises:
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
-            CIMError: CIM_ERR_NOT_FOUND, Interop namespace could not be
-              determined.
+            ModelError: An error with the model implemented by the WBEM server.
         """
         test_classname = 'CIM_Namespace'
         interop_ns = None
@@ -1087,30 +1106,28 @@ class WBEMServer(object):
                 if exc.status_code == CIM_ERR_INVALID_NAMESPACE:
                     # Current namespace does not exist.
                     continue
-                elif exc.status_code in (CIM_ERR_INVALID_CLASS,
-                                         CIM_ERR_NOT_FOUND):
+                if exc.status_code in (CIM_ERR_INVALID_CLASS,
+                                       CIM_ERR_NOT_FOUND):
                     # Class is not implemented, but current namespace exists.
                     interop_ns = ns
                     break
-                else:
-                    # Some other error happened.
-                    raise
-            else:
-                # Namespace class is implemented in the current namespace.
-                # Use the returned namespace name, if possible.
-                ns_names = [p.keybindings['name'] for p in inst_paths]
-                ns_dict = NocaseDict(list(zip(ns_names, ns_names)))
-                try:
-                    interop_ns = ns_dict[ns]
-                except KeyError:
-                    interop_ns = ns
-                break
+                # Some other error happened.
+                raise
+
+            # Namespace class is implemented in the current namespace.
+            # Use the returned namespace name, if possible.
+            ns_names = [p.keybindings['name'] for p in inst_paths]
+            ns_dict = NocaseDict(list(zip(ns_names, ns_names)))
+            try:
+                interop_ns = ns_dict[ns]
+            except KeyError:
+                interop_ns = ns
+            break
         if interop_ns is None:
             # Exhausted the possible namespaces
-            raise CIMError(
-                CIM_ERR_NOT_FOUND,
-                _format("Interop namespace could not be determined "
-                        "(tried {0!A})", self.INTEROP_NAMESPACES),
+            raise ModelError(
+                _format("Interop namespace does not exist (tried {0!A})",
+                        self.INTEROP_NAMESPACES),
                 conn_id=self.conn.conn_id)
         self._interop_ns = interop_ns
 
@@ -1172,11 +1189,7 @@ class WBEMServer(object):
         Raises:
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
-            ModelError: An issue with the model implemented by the WBEM server.
-            CIMError: CIM_ERR_NOT_FOUND, Interop namespace could not be
-              determined.
-            CIMError: CIM_ERR_NOT_FOUND, Namespace class could not be
-              determined.
+            ModelError: An error with the model implemented by the WBEM server.
         """
         ns_insts = None
         ns_classname = None
@@ -1190,17 +1203,15 @@ class WBEMServer(object):
                                        CIM_ERR_NOT_FOUND):
                     # Class is not implemented, try next one.
                     continue
-                else:
-                    # Some other error.
-                    raise
-            else:
-                # Found a namespace class that is implemented.
-                ns_classname = classname
-                break
+                # Some other error.
+                raise
+
+            # Found a namespace class that is implemented.
+            ns_classname = classname
+            break
         if ns_insts is None:
             # Exhausted the possible class names
-            raise CIMError(
-                CIM_ERR_NOT_FOUND,
+            raise ModelError(
                 _format("Namespace class could not be determined "
                         "(tried {0!A})", self.NAMESPACE_CLASSNAMES),
                 conn_id=self.conn.conn_id)
@@ -1233,16 +1244,13 @@ class WBEMServer(object):
         Raises:
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
-            CIMError: CIM_ERR_NOT_FOUND, Interop namespace could not be
-              determined.
-            CIMError: CIM_ERR_NOT_FOUND, Unexpected number of
-              `CIM_ObjectManager` instances.
+            ModelError: An error with the model implemented by the WBEM server.
         """
+        interop_ns = self.interop_ns  # Determines the Interop namespace
         cimom_insts = self._conn.EnumerateInstances(
-            "CIM_ObjectManager", namespace=self.interop_ns)
+            "CIM_ObjectManager", namespace=interop_ns)
         if len(cimom_insts) != 1:
-            raise CIMError(
-                CIM_ERR_NOT_FOUND,
+            raise ModelError(
                 _format("Unexpected number of CIM_ObjectManager instances: "
                         "{0!A} ", [i['ElementName'] for i in cimom_insts]),
                 conn_id=self.conn.conn_id)
@@ -1312,9 +1320,9 @@ class WBEMServer(object):
         Raises:
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
-            CIMError: CIM_ERR_NOT_FOUND, Interop namespace could not be
-              determined.
+            ModelError: An error with the model implemented by the WBEM server.
         """
+        interop_ns = self.interop_ns  # Determines the Interop namespace
         mp_insts = self._conn.EnumerateInstances("CIM_RegisteredProfile",
-                                                 namespace=self.interop_ns)
+                                                 namespace=interop_ns)
         self._profiles = mp_insts
