@@ -67,6 +67,13 @@ operation:
 
   * OpenPegasus
 
+  Because the interpretation of the calculated average and min/max server times
+  becomes incorrect if only a subset of the operations return the server
+  response time, the statistics counting for the server time is suspended if
+  one or more operations do not return the server response time.
+  Resetting the statistics via :meth:`~pywbem.Statistics.reset` clears this
+  condition again.
+
 The difference between client time and server time is the time spent in the
 pywbem client, plus the time spent on the network between client and server.
 
@@ -85,7 +92,7 @@ during periods of time where the statistics are disabled on a connection, are
 simply ignored in the statistics.
 
 For the Iter methods of WBEMConnection (e.g.
-:meth:`~pywbem.WbemConnection.IterEnumerateInstances`), the WBEM operations
+:meth:`~pywbem.WBEMConnection.IterEnumerateInstances`), the WBEM operations
 performed on behalf of them are subject of the statistics, but the Iter methods
 themselves do not show up in the statistics.
 
@@ -146,6 +153,11 @@ from ._utils import _format
 
 __all__ = ['Statistics', 'OperationStatistic']
 
+# pylint: disable=consider-using-min-builtin
+# pylint: disable=consider-using-max-builtin
+#  replaces if statements with something like:
+#  self._reply_len_min = min(_reply_len_min, reply_len)
+
 
 class OperationStatistic(object):
     # pylint: disable=too-many-instance-attributes
@@ -169,24 +181,25 @@ class OperationStatistic(object):
             The statistics container that holds this operation statistic
             object.
 
-          name (string):
+          name (:term:`string`):
             Name of the operation.
         """
         self._container = container
         self._stat_start_time = None
+        self._start_time = None
         self._name = name
+
         self._count = 0
         self._exception_count = 0
+
         self._time_sum = float(0)
         self._time_min = float('inf')
         self._time_max = float(0)
 
+        self._server_time_suspended = False
         self._server_time_sum = float(0)
         self._server_time_min = float('inf')
         self._server_time_max = float(0)
-        self._server_time_stored = False
-
-        self._start_time = None
 
         self._request_len_sum = float(0)
         self._request_len_min = float('inf')
@@ -360,17 +373,19 @@ class OperationStatistic(object):
         """
         Reset the statistics data for this object.
         """
+        self._stat_start_time = None
+
         self._count = 0
         self._exception_count = 0
-        self._stat_start_time = None
+
         self._time_sum = float(0)
         self._time_min = float('inf')
         self._time_max = float(0)
 
+        self._server_time_suspended = False
         self._server_time_sum = float(0)
         self._server_time_min = float('inf')
         self._server_time_max = float(0)
-        self._server_time_stored = False
 
         self._request_len_sum = float(0)
         self._request_len_min = float('inf')
@@ -395,7 +410,7 @@ class OperationStatistic(object):
             if not self._stat_start_time:
                 self._stat_start_time = self._start_time
 
-    def stop_timer(self, request_len, reply_len, server_time=None,
+    def stop_timer(self, request_len=None, reply_len=None, server_time=None,
                    exception=False):
         """
         This is a low-level method is called by pywbem at the end of an
@@ -405,21 +420,28 @@ class OperationStatistic(object):
 
         Parameters:
 
-          request_len (:term:`integer`)
+          request_len (:term:`integer`):
             Size of the HTTP body of the CIM-XML request message, in Bytes.
 
-          reply_len (:term:`integer`)
+          reply_len (:term:`integer`):
             Size of the HTTP body of the CIM-XML response message, in Bytes.
 
-          exception (:class:`py:bool`)
+          exception (:class:`py:bool`):
             Boolean that specifies whether an exception was raised while
             processing the operation.
 
-          server_time (:class:`py:bool`)
+          server_time (:class:`py:bool`):
             Time in seconds that the server optionally returns to the
             client in the HTTP response defining the time from when the
             server received the request to when it started sending the
-            response. If `None`, there is no time from the server.
+            response.
+
+            If `None`, the server did not report server response time. Since
+            this can happen in the middle of a statistics measurement interval
+            if the corresponding config setting of the server was changed,
+            the server time is then reset to 0 and no longer maintained
+            during the remainder of the current statistics measurement interval.
+            A reset of the statistics clears that condition again.
 
         Returns:
 
@@ -437,35 +459,49 @@ class OperationStatistic(object):
         dt = time.time() - self._start_time
         self._start_time = None
         self._count += 1
-        self._time_sum += dt
-        self._request_len_sum += request_len
-        self._reply_len_sum += reply_len
-
         if exception:
             self._exception_count += 1
 
+        self._time_sum += dt
         if dt > self._time_max:
             self._time_max = dt
         if dt < self._time_min:
             self._time_min = dt
 
-        if server_time:
-            self._server_time_stored = True
+        if self._server_time_suspended:
+            # Server time statistics has been suspended. Ignore server time
+            # even if the server returned it for this operation.
+            pass
+        elif server_time is None:
+            # Server did not return server response time for this operation.
+            # Suspend server time statistics and reset the counters.
+            self._server_time_suspended = True
+            self._server_time_sum = float(0)
+            self._server_time_min = float('inf')
+            self._server_time_max = float(0)
+        else:
+            # Server did return server response time for this operation, and
+            # server time statistics has not been suspended. Apply the time
+            # to the counnters.
             self._server_time_sum += server_time
-            if dt > self._server_time_max:
+            if server_time > self._server_time_max:
                 self._server_time_max = server_time
-            if dt < self._server_time_min:
+            if server_time < self._server_time_min:
                 self._server_time_min = server_time
 
-        if request_len > self._request_len_max:
-            self._request_len_max = request_len
-        if request_len < self._request_len_min:
-            self._request_len_min = request_len
+        if request_len is not None:
+            self._request_len_sum += request_len
+            if request_len > self._request_len_max:
+                self._request_len_max = request_len
+            if request_len < self._request_len_min:
+                self._request_len_min = request_len
 
-        if reply_len > self._reply_len_max:
-            self._reply_len_max = reply_len
-        if reply_len < self._reply_len_min:
-            self._reply_len_min = reply_len
+        if reply_len is not None:
+            self._reply_len_sum += reply_len
+            if reply_len > self._reply_len_max:
+                self._reply_len_max = reply_len
+            if reply_len < self._reply_len_min:
+                self._reply_len_min = reply_len
 
         return dt
 
@@ -493,19 +529,26 @@ class OperationStatistic(object):
             "max_reply_len={s.max_reply_len!A})",
             s=self)
 
-    _formatted_header_w_svr = \
-        'Count Excep         ClientTime              ServerTime        ' \
-        '     RequestLen                ReplyLen       Operation\n' \
-        '        Cnt     Avg     Min     Max     Avg     Min     Max   ' \
-        ' Avg    Min    Max      Avg      Min      Max\n'
+    @staticmethod
+    def formatted_header(include_server_time, include_lengths):
+        """
+        Return a two-line header.
+        """
+        ret_lines = [
+            'Count ExcCnt          Time [s]      ',
+            '                Avg     Min     Max ',
+        ]
+        if include_server_time:
+            ret_lines[0] += '       ServerTime [s]   '
+            ret_lines[1] += '    Avg     Min     Max '
+        if include_lengths:
+            ret_lines[0] += '       RequestLen [B]            ReplyLen [B]   '
+            ret_lines[1] += '   Avg    Min    Max      Avg      Min      Max '
+        ret_lines[0] += 'Operation\n'
+        ret_lines[1] += '\n'
+        return ''.join(ret_lines)
 
-    _formatted_header = \
-        'Count Excep         ClientTime        ' \
-        '     RequestLen              ReplyLen       Operation\n' \
-        '        Cnt     Avg     Min     Max   ' \
-        ' Avg    Min    Max    Avg      Min      Max\n'
-
-    def formatted(self, include_server_time):
+    def formatted(self, include_server_time, include_lengths):
         """
         Return a formatted one-line string with the statistics
         values for the operation for which this statistics object
@@ -514,33 +557,21 @@ class OperationStatistic(object):
         This is a low-level method that is called by
         :meth:`pywbem.Statistics.formatted`.
         """
-        if include_server_time:  # pylint: disable=no-else-return
-            return ('{0:5d} {1:5d} '
-                    '{2:7.3f} {3:7.3f} {4:7.3f} '
-                    '{5:7.3f} {6:7.3f} {7:7.3f} '
-                    '{8:6.0f} {9:6.0f} {10:6.0f} '
-                    '{11:8.0f} {12:8.0f} {13:8.0f} {14}\n'.
-                    format(self.count, self.exception_count,
-                           self.avg_time, self.min_time, self.max_time,
-                           self.avg_server_time, self.min_server_time,
-                           self.max_server_time,
-                           self.avg_request_len, self.min_request_len,
-                           self.max_request_len,
-                           self.avg_reply_len, self.min_reply_len,
-                           self.max_reply_len,
-                           self.name))
-        else:
-            return ('{0:5d} {1:5d} '
-                    '{2:7.3f} {3:7.3f} {4:7.3f} '
-                    '{5:6.0f} {6:6.0f} {7:6.0f} '
-                    '{8:6.0f} {9:8.0f} {10:8.0f} {11}\n'.
-                    format(self.count, self.exception_count,
-                           self.avg_time, self.min_time, self.max_time,
-                           self.avg_request_len, self.min_request_len,
-                           self.max_request_len,
-                           self.avg_reply_len, self.min_reply_len,
-                           self.max_reply_len,
-                           self.name))
+        ret = ('{0:5d} {1:5d} {2:7.3f} {3:7.3f} {4:7.3f} '.
+               format(self.count, self.exception_count,
+                      self.avg_time, self.min_time, self.max_time))
+        if include_server_time:
+            ret += ('{0:7.3f} {1:7.3f} {2:7.3f} '.
+                    format(self.avg_server_time, self.min_server_time,
+                           self.max_server_time))
+        if include_lengths:
+            ret += ('{0:6.0f} {1:6.0f} {2:6.0f} '
+                    '{3:8.0f} {4:8.0f} {5:8.0f} '.
+                    format(self.avg_request_len, self.min_request_len,
+                           self.max_request_len, self.avg_reply_len,
+                           self.min_reply_len, self.max_reply_len))
+        ret += '{0}\n'.format(self.name)
+        return ret
 
 
 class Statistics(object):
@@ -565,6 +596,8 @@ class Statistics(object):
     The enablement state of the :class:`~pywbem.Statistics` object is
     controlled by the statistics enablement state of the connection it belongs
     to (see :meth:`pywbem.WBEMConnection.stats_enabled`)
+
+    This class can also be used as a context manager.
     """
 
     def __init__(self, enable=False):
@@ -579,6 +612,54 @@ class Statistics(object):
         self._op_stats = {}
         self._disabled_stats = OperationStatistic(self, "disabled")
 
+        # Used in context manager (which supports nesting)
+        self._cm_stack = []  # items: OperationStatistic object
+        self._cm_name = None  # stored only between __call__() and __enter__()
+
+    def __enter__(self):
+        """
+        Enter method when the class is used as a context manager.
+
+        Starts the operation statistics for the name that was previously set
+        via a call::
+
+            stats = Statistics()
+            with stats(name='bla'):
+                # do something
+
+        The class supports nesting of context managers::
+
+            stats = Statistics()
+            with stats(name='bla1'):
+                # do something
+                for i in ...:
+                    with stats(name='bla2'):
+                        # do something
+        """
+        name = self._cm_name
+        self._cm_name = None
+        op_stat = self.start_timer(name)
+        self._cm_stack.append(op_stat)
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        """
+        Exit method when the class is used as a context manager.
+
+        Stops the operation statistics that was started in the enter method.
+        """
+        op_stat = self._cm_stack.pop()
+        op_stat.stop_timer()
+        return False  # re-raise any exceptions
+
+    def __call__(self, name):
+        """
+        This allows the `name` parameter to be passed when the class is used
+        as a context manager.
+        """
+        self._cm_name = name
+        return self
+
     @property
     def enabled(self):
         """
@@ -610,7 +691,7 @@ class Statistics(object):
 
         Parameters:
 
-          name (string):
+          name (:term:`string`):
             Name of the operation.
 
         Returns:
@@ -630,7 +711,7 @@ class Statistics(object):
 
         Parameters:
 
-          name (string):
+          name (:term:`string`):
             Name of the operation.
 
         Returns:
@@ -661,15 +742,15 @@ class Statistics(object):
           - stats (:class:`~pywbem.OperationStatistic`): Time statistics for
             the operation
         """
-        return copy.deepcopy(self._op_stats).items()
+        return list(copy.deepcopy(self._op_stats).items())
 
     def __repr__(self):
         """
         Return a human readable display of the contents, for debug purposes.
         """
         ret = "Statistics(\n"
-        for name in self._op_stats:
-            ret += _format("  {0!A}\n", self._op_stats[name])
+        for stats_value in self._op_stats.values():
+            ret += _format("  {0!A}\n", stats_value)
         ret += ")"
         return ret
 
@@ -680,24 +761,27 @@ class Statistics(object):
         The operations are sorted by decreasing average time.
 
         The three columns for `ServerTime` are included only if the WBEM server
-        has returned WBEM server response times.
+        has returned WBEM server response times for all operations.
+
+        The six columns for `RequestLen` and `ReplyLen` are included only if
+        they are non-zero (this allows using this class for other purposes).
 
         Example if statistics are enabled::
 
-            Statistics (times in seconds, lengths in Bytes):
-            Count Excep         ClientTime              ServerTime             RequestLen                ReplyLen       Operation
-                    Cnt     Avg     Min     Max     Avg     Min     Max    Avg    Min    Max      Avg      Min      Max
+            Statistics:
+            Count ExcCnt         Time [s]             ServerTime [s]        RequestLen [B]            ReplyLen [B]      Operation
+                            Avg     Min     Max     Avg     Min     Max    Avg    Min    Max      Avg      Min      Max
                 3     0   0.234   0.100   0.401   0.204   0.080   0.361   1233   1000   1500    26667    20000    35000 EnumerateInstances
                 1     0   0.100   0.100   0.100   0.080   0.080   0.080   1200   1200   1200    22000    22000    22000 EnumerateInstanceNames
                 . . .
 
         Example if statistics are disabled::
 
-            Statistics (times in seconds, lengths in Bytes):
+            Statistics:
             Disabled
         """  # noqa: E501
         # pylint: enable=line-too-long
-        ret = "Statistics (times in seconds, lengths in Bytes):\n"
+        ret = "Statistics:\n"
         if self.enabled:
             snapshot = sorted(self.snapshot(),
                               key=lambda item: item[1].avg_time,
@@ -705,18 +789,20 @@ class Statistics(object):
 
             # Test to see if any server time is non-zero
             include_svr = False
-            for name, stats in snapshot:  # pylint: disable=unused-variable
+            include_len = False
+            for _, stats in snapshot:
                 # pylint: disable=protected-access
-                if stats._server_time_stored:
+                if not stats._server_time_suspended:
                     include_svr = True
-            # pylint: disable=protected-access
-            if include_svr:
-                ret += OperationStatistic._formatted_header_w_svr
-            else:
-                ret += OperationStatistic._formatted_header
+                # pylint: disable=protected-access
+                if stats._request_len_sum > 0 or stats._reply_len_sum > 0:
+                    include_len = True
+
+            ret += OperationStatistic.formatted_header(
+                include_svr, include_len)
+            for _, stats in snapshot:
+                ret += stats.formatted(include_svr, include_len)
 
-            for name, stats in snapshot:  # pylint: disable=unused-variable
-                ret += stats.formatted(include_svr)
         else:
             ret += "Disabled"
         return ret.strip()
