@@ -87,10 +87,10 @@ class CIMContentHandler(xml.sax.ContentHandler):
         self.element = []
 
     def startDocument(self):
-        assert self.elements == []
+        assert not self.elements
 
     def endDocument(self):
-        assert self.elements == []
+        assert not self.elements
         self.root = self.element
 
     def startElement(self, name, attrs):
@@ -156,15 +156,10 @@ def xml_to_tupletree_sax(xml_string, meaning, conn_id=None):
 
     handler = CIMContentHandler()
 
-    # The following conversion to a byte string is required for two reasons:
-    # 1. xml.sax.parseString() raises UnicodeEncodeError for unicode strings
-    #    that contain any non-ASCII characters (despite its Python 2.7
-    #    documentation which states that would be supported).
-    # 2. The SAX parser in Python 2.6 and 3.4 (pywbem does not support 3.1 -
-    #    3.3) does not accept unicode strings, raising:
-    #      SAXParseException: "<unknown>:1:1: not well-formed (invalid token)"
-    #    or:
-    #      TypeError: 'str' does not support the buffer interface
+    # The following conversion to a byte string is required because:
+    # * xml.sax.parseString() raises UnicodeEncodeError for unicode strings
+    #   that contain any non-ASCII characters (despite its Python 2.7
+    #   documentation which states that would be supported).
     xml_string = _ensure_bytes(xml_string)
 
     try:
@@ -343,15 +338,23 @@ def check_invalid_utf8_sequences(utf8_string, meaning, conn_id=None):
     (3) The Python escapes ``\\u`` and ``\\U`` used in literal strings can
         represent the surrogate code points (as well as all other code points,
         regardless of whether they are assigned to Unicode characters).
+        That is the case in both Python 2 and Python 3.
 
-    (4) The Python `encode()` and `decode()` functions successfully
-        translate the surrogate code points back and forth for encoding UTF-8.
+    (4) In Python 2, the `unicode.encode()` and `str.decode()` methods
+        successfully translate surrogate code points back and forth for
+        encoding UTF-8, tolerating invalid surrogate sequences.
 
         For example, ``b'\\xed\\xb0\\x80'.decode("utf-8") = u'\\udc00'``.
 
-    (5) Because Python supports the encoding and decoding of UTF-8 sequences
+        In Python 3, the `str.encode()` and `bytes.decode()` methods raise
+        UnicodeEncodeError / UnicodeDecodeError for invalid surrogate
+        sequences. However, the `codecs.encode()` and `codecs.decode()`
+        methods have an error handler 'surrogatepass' which tolerates
+        invalid surrogate sequences.
+
+    (5) Because Python 2 supports the encoding and decoding of UTF-8 sequences
         also for the surrogate code points, the "narrow" Unicode build of
-        Python can be (mis-)used to transport each surrogate unit separately
+        Python 2 can be (mis-)used to transport each surrogate unit separately
         encoded in (ill-formed) UTF-8.
 
         For example, code point U+10122 can be (illegally) created from a
@@ -378,7 +381,7 @@ def check_invalid_utf8_sequences(utf8_string, meaning, conn_id=None):
     # before the str type gets decoded to unicode, because afterwards
     # surrogates produced from ill-formed UTF-8 cannot be distinguished from
     # legally produced surrogates (for code points above U+FFFF).
-    ifs_list = list()
+    ifs_list = []
     for m in _ILL_FORMED_UTF8_RE.finditer(utf8_string):
         ifs_pos = m.start(1)
         ifs_seq = m.group(1)
@@ -490,7 +493,7 @@ def check_invalid_xml_chars(xml_string, meaning, conn_id=None):
 
     # Check for Unicode characters that cannot legally be represented as XML
     # characters.
-    ixc_list = list()
+    ixc_list = []
     last_ixc_pos = -2
     for m in _ILLEGAL_XML_CHARS_RE.finditer(xml_string):
         ixc_pos = m.start(1)
