@@ -34,6 +34,7 @@ import six
 from ._cim_types import CIMInt, type_from_name
 from ._cim_obj import CIMProperty, CIMMethod, CIMParameter
 from ._utils import _format, _integerValue_to_int
+from ._exceptions import ModelError
 
 __all__ = ['ValueMapping']
 
@@ -47,6 +48,8 @@ class ValueMapping(object):
     with the `ValueMap` and `Values` qualifiers, and the corresponding values
     of the `Values` qualifier, in both directions.
 
+    The CIM element may be a scalar or an array.
+
     This is done by retrieving the CIM class definition defining the CIM
     element in question, and by inspecting its `ValueMap` and `Values`
     qualifiers.
@@ -67,6 +70,12 @@ class ValueMapping(object):
     supported (decimal, binary, octal, hexadecimal), consistent with the
     definition of the `ValueMap` qualifier in :term:`DSP0004`.
 
+    If the sizes of the `Values` and `ValueMap` arrays differ, the default
+    behavior is that a :exc:`~pywbem.ModelError` exception will be raised.
+    Alternatively, the behavior can be changed to have the `Values` array
+    adjusted to the size of the `ValueMap` array, by filling in any missing
+    items at the end with a default value, or by truncating any extra items.
+
     Example:
 
       Given the following definition of a property in MOF:
@@ -190,6 +199,7 @@ class ValueMapping(object):
         self._propname = None
         self._methodname = None
         self._parametername = None
+        self._values_default = None
 
         self._element_obj = None
 
@@ -202,13 +212,16 @@ class ValueMapping(object):
         self._v2b_dict = {}  # values: bin (int or tuple)
 
     @classmethod
-    def for_property(cls, server, namespace, classname, propname):
+    def for_property(cls, server, namespace, classname, propname,
+                     values_default=None):
         # pylint: disable=line-too-long
         """
         Factory method that returns a new :class:`~pywbem.ValueMapping`
         instance that maps CIM property values to the `Values` qualifier
         defined on that property.
 
+        The CIM property may be a scalar or an array.
+
         If a `Values` qualifier is defined but no `ValueMap` qualifier, a
         default of 0-based consecutive numbers is applied (that is the default
         defined in :term:`DSP0004`).
@@ -230,6 +243,17 @@ class ValueMapping(object):
             Name of the CIM property that defines the `Values` / `ValueMap`
             qualifiers.
 
+          values_default (`None` or :term:`string`):
+            Controls the handling of different sizes of the `Values` and
+            `ValueMap` arrays:
+
+            - If `None` (default), a :exc:`~pywbem.ModelError` exception will be
+              raised for different sizes of the `Values` and `ValueMap` arrays.
+            - If a string value is specified, the `Values` array will be
+              adjusted to the size of the `ValueMap` array, by filling in any
+              missing items with the specified default value at the end, or by
+              removing any extra items.
+
         Returns:
 
             The new :class:`~pywbem.ValueMapping` instance.
@@ -238,10 +262,11 @@ class ValueMapping(object):
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
             KeyError: The CIM property does not exist in the CIM class.
-            TypeError: The CIM property is not integer-typed.
+            ModelError: The CIM property is not integer-typed.
             ValueError: No `Values` qualifier defined on the CIM property.
-            ValueError: Invalid integer representation in `ValueMap` qualifier
+            ModelError: Invalid integer representation in `ValueMap` qualifier
               defined on the CIM property.
+            ModelError: `Values` and `ValueMap` arrays differ in size.
         """  # noqa: E501
 
         conn = server
@@ -263,12 +288,14 @@ class ValueMapping(object):
                         "{2!A}", classname, namespace, propname))
 
         new_vm = cls._create_for_element(property_obj, conn, namespace,
-                                         classname, propname=propname)
+                                         classname, propname=propname,
+                                         values_default=values_default)
 
         return new_vm
 
     @classmethod
-    def for_method(cls, server, namespace, classname, methodname):
+    def for_method(cls, server, namespace, classname, methodname,
+                   values_default=None):
         # pylint: disable=line-too-long
         """
         Factory method that returns a new :class:`~pywbem.ValueMapping`
@@ -295,6 +322,17 @@ class ValueMapping(object):
             Name of the CIM method that defines the `Values` / `ValueMap`
             qualifiers.
 
+          values_default (`None` or :term:`string`):
+            Controls the handling of different sizes of the `Values` and
+            `ValueMap` arrays:
+
+            - If `None` (default), a :exc:`~pywbem.ModelError` exception will be
+              raised for different sizes of the `Values` and `ValueMap` arrays.
+            - If a string value is specified, the `Values` array will be
+              adjusted to the size of the `ValueMap` array, by filling in any
+              missing items with the specified default value at the end, or by
+              removing any extra items.
+
         Returns:
 
             The new :class:`~pywbem.ValueMapping` instance.
@@ -303,10 +341,11 @@ class ValueMapping(object):
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
             KeyError: The CIM method does not exist in the CIM class.
-            TypeError: The CIM method is not integer-typed.
+            ModelError: The CIM method is not integer-typed.
             ValueError: No `Values` qualifier defined on the CIM method.
-            ValueError: Invalid integer representation in `ValueMap` qualifier
+            ModelError: Invalid integer representation in `ValueMap` qualifier
               defined on the CIM method.
+            ModelError: `Values` and `ValueMap` arrays differ in size.
         """  # noqa: E501
 
         conn = server
@@ -328,19 +367,22 @@ class ValueMapping(object):
                         classname, namespace, methodname))
 
         new_vm = cls._create_for_element(method_obj, conn, namespace,
-                                         classname, methodname=methodname)
+                                         classname, methodname=methodname,
+                                         values_default=values_default)
 
         return new_vm
 
     @classmethod
     def for_parameter(cls, server, namespace, classname, methodname,
-                      parametername):
+                      parametername, values_default=None):
         # pylint: disable=line-too-long
         """
         Factory method that returns a new :class:`~pywbem.ValueMapping`
         instance that maps CIM parameter values to the `Values` qualifier
         defined on that parameter.
 
+        The CIM parameter may be a scalar or an array.
+
         If a `Values` qualifier is defined but no `ValueMap` qualifier, a
         default of 0-based consecutive numbers is applied (that is the default
         defined in :term:`DSP0004`).
@@ -364,6 +406,17 @@ class ValueMapping(object):
             Name of the CIM parameter that defines the `Values` / `ValueMap`
             qualifiers.
 
+          values_default (`None` or :term:`string`):
+            Controls the handling of different sizes of the `Values` and
+            `ValueMap` arrays:
+
+            - If `None` (default), a :exc:`~pywbem.ModelError` exception will be
+              raised for different sizes of the `Values` and `ValueMap` arrays.
+            - If a string value is specified, the `Values` array will be
+              adjusted to the size of the `ValueMap` array, by filling in any
+              missing items with the specified default value at the end, or by
+              removing any extra items.
+
         Returns:
 
             The new :class:`~pywbem.ValueMapping` instance.
@@ -373,10 +426,11 @@ class ValueMapping(object):
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
             KeyError: The CIM method does not exist in the CIM class.
             KeyError: The CIM parameter does not exist in the CIM method.
-            TypeError: The CIM parameter is not integer-typed.
+            ModelError: The CIM parameter is not integer-typed.
             ValueError: No `Values` qualifier defined on the CIM parameter.
-            ValueError: Invalid integer representation in `ValueMap` qualifier
+            ModelError: Invalid integer representation in `ValueMap` qualifier
               defined on the CIM parameter.
+            ModelError: `Values` and `ValueMap` arrays differ in size.
         """  # noqa: E501
 
         conn = server
@@ -406,7 +460,8 @@ class ValueMapping(object):
 
         new_vm = cls._create_for_element(parameter_obj, conn, namespace,
                                          classname, methodname=methodname,
-                                         parametername=parametername)
+                                         parametername=parametername,
+                                         values_default=values_default)
 
         return new_vm
 
@@ -431,7 +486,7 @@ class ValueMapping(object):
 
         Raises:
 
-            ValueError: Invalid integer representation in `ValueMap` qualifier
+            ModelError: Invalid integer representation in `ValueMap` qualifier
               defined on the CIM element.
         """
         values_str = values_list[i]
@@ -466,10 +521,12 @@ class ValueMapping(object):
         return (lo, hi, values_str)
 
     def _to_int(self, val_str):
-        """Conver val_str to an integer or raise ValueError"""
+        """
+        Convert val_str to an integer or raise ModelError
+        """
         val = _integerValue_to_int(val_str)
         if val is None:
-            raise ValueError(
+            raise ModelError(
                 _format("The value-mapped {0} has an invalid integer "
                         "representation in a ValueMap entry: {1!A}",
                         self._element_str(), val_str))
@@ -477,12 +534,15 @@ class ValueMapping(object):
 
     @classmethod
     def _create_for_element(cls, element_obj, conn, namespace, classname,
-                            propname=None, methodname=None, parametername=None):
+                            propname=None, methodname=None, parametername=None,
+                            values_default=None):
         # pylint: disable=line-too-long
         """
         Return a new :class:`~pywbem.ValueMapping` instance for the specified
         CIM element.
 
+        The CIM element may be a scalar or an array.
+
         If a `Values` qualifier is defined but no `ValueMap` qualifier, a
         default of 0-based consecutive numbers is applied (that is the default
         defined in :term:`DSP0004`).
@@ -499,20 +559,32 @@ class ValueMapping(object):
             Name of the CIM namespace containing the class.
 
           classname (:term:`string`):
-            Name of the CIM class exposing the method. The method can be
-            defined in that class or inherited into that class.
+            Name of the CIM class exposing the property or method. The property
+            or method can be defined in that class or inherited into that class.
 
           propname (:term:`string`):
             Name of the CIM property that defines the `Values` / `ValueMap`
             qualifiers.
 
           methodname (:term:`string`):
-            Name of the CIM method that has the parameter.
+            Name of the CIM method that defines the `Values` / `ValueMap`
+            qualifiers on its return value, or that has the parameter.
 
           parametername (:term:`string`):
             Name of the CIM parameter that defines the `Values` / `ValueMap`
             qualifiers.
 
+          values_default (`None` or :term:`string`):
+            Controls the handling of different sizes of the `Values` and
+            `ValueMap` arrays:
+
+            - If `None` (default), a :exc:`~pywbem.ModelError` exception will be
+              raised for different sizes of the `Values` and `ValueMap` arrays.
+            - If a string value is specified, the `Values` array will be
+              adjusted to the size of the `ValueMap` array, by filling in any
+              missing items with the specified default value at the end, or by
+              removing any extra items.
+
         Returns:
 
             The created :class:`~pywbem.ValueMapping` instance for the specified
@@ -520,10 +592,11 @@ class ValueMapping(object):
 
         Raises:
 
-            TypeError: The CIM element is not integer-typed.
+            ModelError: The CIM element is not integer-typed.
             ValueError: No `Values` qualifier defined on the CIM element.
-            ValueError: Invalid integer representation in `ValueMap` qualifier
+            ModelError: Invalid integer representation in `ValueMap` qualifier
               defined on the CIM element.
+            ModelError: `Values` and `ValueMap` arrays differ in size.
         """  # noqa: E501
         # pylint: enable=line-too-long
 
@@ -537,6 +610,7 @@ class ValueMapping(object):
         vm._propname = propname
         vm._methodname = methodname
         vm._parametername = parametername
+        vm._values_default = values_default
 
         try:
             typename = element_obj.type  # Property, Parameter
@@ -546,7 +620,8 @@ class ValueMapping(object):
         cimtype = type_from_name(typename)
 
         if not issubclass(cimtype, CIMInt):
-            raise TypeError(
+            # This test works for both scalar and array-typed elements
+            raise ModelError(
                 _format("The value-mapped {0} is not integer-typed, but "
                         "has CIM type: {1}", vm._element_str(), typename))
 
@@ -556,39 +631,60 @@ class ValueMapping(object):
             raise ValueError(
                 _format("The value-mapped {0} has no Values qualifier "
                         "defined", vm._element_str()))
-        values_list = values_qual.value
+        values_list = list(values_qual.value)  # may be modified
 
         valuemap_qual = element_obj.qualifiers.get('ValueMap', None)
         if valuemap_qual is None:
             # DSP0004 defines a default of consecutive index numbers
-            vm._b2v_single_dict = dict(zip(range(0, len(values_list)),
-                                           values_list))
-            vm._b2v_range_tuple_list = []
-            vm._b2v_unclaimed = None
-            vm._v2b_dict = OrderedDict(zip(values_list,
-                                           range(0, len(values_list))))
+            valuemap_list = ["{}".format(v) for v in range(0, len(values_list))]
         else:
-            vm._b2v_single_dict = {}
-            vm._b2v_range_tuple_list = []
-            vm._b2v_unclaimed = None
-            vm._v2b_dict = OrderedDict()
             valuemap_list = valuemap_qual.value
-            for i, valuemap_str in enumerate(valuemap_list):
-                values_str = values_list[i]
-                if valuemap_str == '..':
-                    vm._b2v_unclaimed = values_str
-                    vm._v2b_dict[values_str] = None
+
+        # Verify and adjust the valuemap and values arrays
+        values_size = len(values_list)
+        valuemap_size = len(valuemap_list)
+        if valuemap_size > values_size:
+            valuemap_extra = valuemap_list[values_size:]
+            if values_default is None:
+                raise ModelError(
+                    _format("The value-mapped {0} has a ValueMap "
+                            "qualifier that specifies more items than "
+                            "its Values qualifier: {1}",
+                            vm._element_str(), valuemap_extra))
+            # Fill the missing Values items with the default value
+            values_list.extend([values_default] * len(valuemap_extra))
+        if valuemap_size < values_size:
+            values_extra = values_list[valuemap_size:]
+            if values_default is None:
+                raise ModelError(
+                    _format("The value-mapped {0} has a Values "
+                            "qualifier that specifies more items than "
+                            "its ValueMap qualifier: {1}",
+                            vm._element_str(), values_extra))
+            # Truncate the extra Values items
+            del values_list[len(values_extra):]
+
+        # Perform data initialization of the ValueMapping object
+        vm._b2v_single_dict = {}
+        vm._b2v_range_tuple_list = []
+        vm._b2v_unclaimed = None
+        vm._v2b_dict = OrderedDict()
+        for i, valuemap_str in enumerate(valuemap_list):
+            values_str = values_list[i]
+            if valuemap_str == '..':
+                vm._b2v_unclaimed = values_str
+                vm._v2b_dict[values_str] = None
+            else:
+                lo, hi, values_str = vm._values_tuple(
+                    i, valuemap_list, values_list, cimtype)
+                if lo == hi:
+                    # single value
+                    vm._b2v_single_dict[lo] = values_str
+                    vm._v2b_dict[values_str] = lo
                 else:
-                    lo, hi, values_str = vm._values_tuple(
-                        i, valuemap_list, values_list, cimtype)
-                    if lo == hi:
-                        # single value
-                        vm._b2v_single_dict[lo] = values_str
-                        vm._v2b_dict[values_str] = lo
-                    else:
-                        # value range
-                        vm._b2v_range_tuple_list.append((lo, hi, values_str))
-                        vm._v2b_dict[values_str] = (lo, hi)
+                    # value range
+                    vm._b2v_range_tuple_list.append((lo, hi, values_str))
+                    vm._v2b_dict[values_str] = (lo, hi)
 
         return vm
 
@@ -622,6 +718,7 @@ class ValueMapping(object):
             "_propname={s._propname!A}, "
             "_methodname={s._methodname!A}, "
             "_parametername={s._parametername!A}, "
+            "_values_default={s._values_default!A}, "
             "_element_obj={s._element_obj!A}, "
             "_b2v_single_dict={s._b2v_single_dict!A}, "
             "_b2v_range_tuple_list={s._b2v_range_tuple_list!A}, "
@@ -678,6 +775,20 @@ class ValueMapping(object):
         """
         return self._parametername
 
+    @property
+    def values_default(self):
+        """
+        `None` or :term:`string`: Controls the handling of different sizes of
+        the `Values` and `ValueMap` arrays:
+
+        - If `None`, a :exc:`~pywbem.ModelError` exception will be raised for
+          different sizes of the `Values` and `ValueMap` arrays.
+        - If a string value, the `Values` array will be adjusted to the size of
+          the `ValueMap` array, by filling in any missing items with that
+          string value at the end, or by removing any extra items.
+        """
+        return self._values_default
+
     @property
     def element(self):
         # pylint: disable=line-too-long
@@ -688,19 +799,62 @@ class ValueMapping(object):
         return self._element_obj
 
     def tovalues(self, element_value):
+        # pylint: disable=line-too-long
         """
-        Return the `Values` string for an element value, based upon this value
-        mapping.
+        Return the `Values` string(s) for an element value, based upon this
+        value mapping.
+
+        The element value may be a single value or list/tuple of values and the
+        return value will be a single string or list of strings, respectively.
+        An element value of `None` causes `None` to be returned.
+
+        The passing of array values or scalar values does not need to match
+        whether the element is array-typed or scalar-typed. For example, there
+        may be a need to have the loop through a list of values of an
+        array-typed element on the caller's side, invoking this method in the
+        loop with a single value. As another example, the method may be used
+        to translate a list of possible values for a scalar-typed element
+        in one call to this method by passing them as a list.
+
+        Parameters:
+
+          element_value (:term:`integer` or :class:`~pywbem.CIMInt` or list/tuple thereof):
+            The value(s) of the CIM element. May be `None`.
+
+        Returns:
+
+          :term:`string` or list of :term:`string`:
+            The `Values` string(s) for the element value.
+            This is:
+            * a single string, if the element value was a single value
+            * a list of strings, if the element value was a list/tuple of values
+            * `None`, if the element value was `None`
+
+        Raises:
+
+          ValueError: Element value outside of the set defined by `ValueMap`.
+          TypeError: Element value is not an integer type.
+        """  # noqa: E501
+        if element_value is None:
+            return None
+        if isinstance(element_value, (list, tuple)):
+            return [self._tovalues_single(ev) for ev in element_value]
+        return self._tovalues_single(element_value)
+
+    def _tovalues_single(self, element_value):
+        """
+        Return the `Values` string for a single element value, based upon this
+        value mapping.
 
         Parameters:
 
           element_value (:term:`integer` or :class:`~pywbem.CIMInt`):
-            The value of the CIM element (property, method, parameter).
+            The single integer value of the CIM element. Must not be `None`.
 
         Returns:
 
           :term:`string`:
-            The `Values` string for the element value.
+            The single `Values` string for the element value.
 
         Raises:
 
@@ -740,6 +894,11 @@ class ValueMapping(object):
         Return the integer value or values for a `Values` string, based upon
         this value mapping.
 
+        Due to the complexity of its return value, this method only supports a
+        single `Values` string at a time. It does support array-typed elements,
+        though. Thus, if multiple `Values` strings need to be translated, this
+        method must be invoked once for each value to be translated.
+
         Any returned integer value is represented as the CIM type of the
         element (e.g. :class:`~pywbem.Uint16`).
 
@@ -757,7 +916,8 @@ class ValueMapping(object):
         Parameters:
 
           values_str (:term:`string`):
-            The `Values` string for the element value.
+            The `Values` string for the (single) element value. Must not be
+            `None`.
 
         Returns:
 
