@@ -15,16 +15,18 @@
 #
 #
 """
-A DMTF CIM schema is the collection of CIM qualifier declarations and CIM
-classes managed by the DMTF and released as a single tested, coherent package
-with a defined major, minor, update version number.  The MOF representation of
+A CIM schema is the collection of CIM qualifier declarations and CIM
+classes and available  a single tested, coherent package
+with a defined major, minor, update version number. The DMTF regularly
+releases updates to the DMTF CIM schema that represent all of the classes
+approved by the DMTF. The MOF representation of
 a DMTF schema release is packaged as a single zip file and is available on the
 DMTF web site ( https://www.dmtf.org/standards/cim ).
 
-Because the DMTF CIM schemas are the source of most of the classes that a WBEM
-server normally implements it was important to create a simple mechanism
-to include the DMTF CIM classes and qualifier declarations in the
-:class:`~pywbem_mock.FakedWBEMConnection` mock repository.
+Because CIM schemas are the source of most of the classes that a WBEM server
+normally implements it was important to create a simple mechanism to include
+the DMTF CIM classes and qualifier declarations from a CIM schema in the
+CIM repository of :class:`~pywbem_mock.FakedWBEMConnection`.
 
 The :class:`~pywbem_mock.DMTFCIMSchema` class downloads the DMTF CIM schema
 zip file defined by version from the DMTF web site into a defined directory,
@@ -36,7 +38,7 @@ classes in the CIM schema having unique class names; this is always true for
 DMTF CIM schema releases to assure that they can be installed in the same
 namespace in a WBEM server repository.
 
-Once a DMTF CIM schema is extracted into the individual MOF files it consumes
+Once aCIM schema is extracted into the individual MOF files it consumes
 considerable space since it expands to almost 3000 files.  Therefore it is
 logical to remove all of the individual MOF files when not being used and also
 if the schema information is to be committed to be used in testing in an
@@ -60,20 +62,23 @@ experimental schema depending on the value of the `use_experimental`
 parameter.
 
 Because it is usually necessary to compile only a subset of the DMTF CIM
-classes into a mock repository for any test, the
+classes into a CIM repository for any test, the
 :class:`~pywbem_mock.DMTFCIMSchema` class provides a
-:meth:`~pywbem_mock.FakedWBEMConnection.build_schema_mof` method to create a
-MOF include file that includes only a subset of the classes in the downloaded
-DMTF CIM Schema. The task of listing classes to be compiled is futher
+:meth:`~pywbem_mock.DMTFCIMSchema.build_schema_mof` method to create a
+list of MOF pragmans that includes only a subset of the classes in the
+downloaded CIM Schema. The task of listing classes to be compiled is futher
 simplified because the pywbem MOF compiler searches the DMTF schema for
 prerequisite classes (superclasses, reference classes, and EmbeddedInstance
 classes) so that generally only the leaf CIM classes required for the
 repository need to be in the class list. This speeds up the process of
 compiling DMTF CIM classes for any test significantly.
 """
+
 import os
+import io
 from zipfile import ZipFile
 import shutil
+import re
 import six
 try:
     from urllib.request import urlopen
@@ -86,15 +91,110 @@ from pywbem._utils import _format
 __all__ = ['DMTFCIMSchema']
 
 
+def build_schema_mof(class_names, schema_pragma_file):
+    """
+    Build a string that includes the ``#include pragma`` statements for the
+    schema CIM classes defined in `class_names` using the CIM schema defined by
+    this object.
+
+    The class names in `class_names` can be just leaf classes. The pywbem
+    MOF compiler will search for dependent classes including superclasses,
+    and other classes referenced as the defined classes are compiled.
+
+    It builds a compilable MOF string in the form::
+
+        #pragma locale ("en_US")
+        #pragma include ("System/CIM_ComputerSystem.mof")
+        ...
+
+    with a ``#pragma include`` for each classname in the `class_names`
+    list.
+
+    The resulting set of #pragma statements is in the same order as the
+    original pragmas.  In some cases that could be important because the
+    DMTF list is ordered to missing dependencies and conflicts between
+    class mof compiles.
+
+    Parameters:
+
+      class_names (:term:`string` or list of :term:`string`):
+        These must be class names of classes in the DMTF CIM schema represented
+        by the instance of :class:`~pywbem_mock.DMTFCIMSchema` object. This
+        parameter can be a string if there is only a single class name to be
+        included.
+
+        If the returned string is compiled, the MOF compiler will search the
+        directory defined by the path component of `schema_pragma_file` for
+        classes referenced by each for class in this list including super
+        classes, classed defined by reference properties, and classes defined
+        by an embedded-object.
+
+    schema_pragma_file (:term:`string`):
+        File path defining a CIM schema pragma file for the set of
+        CIM classes that make up a schema such as the DMTF schema.
+        This file must contain a pragma statement for each of the
+        classes defined in the schema.
+
+    Returns:
+        :term:`string`: Valid MOF containing pragma statements from the
+        `CIM schema pragma file` for all of the classes in `schema_classes`.
+
+    Raises:
+        ValueError: If any of the classnames in `schema_classes` are not in
+        the `CIM schema pragma file`.
+    """
+    if isinstance(class_names, six.string_types):
+        class_names = [class_names]
+
+    schema_lines = []
+    with io.open(schema_pragma_file, 'r', encoding='utf-8') as f:
+        schema_lines = f.readlines()
+
+    # Build list  classname/line number pairs
+    classname_pattern = ' *#.*include.*/(.*)\\.mof'
+    class_names_lc = [cln.lower() for cln in class_names]
+    lines_list = []
+    found_classes = []
+    for line_no, line in enumerate(schema_lines, 0):
+        result = re.search(classname_pattern, line)
+        if result:
+            cln = result.group(1)
+            if cln.lower() in class_names_lc:
+                lines_list.append(line_no)
+                found_classes.append(cln)
+
+    # Create list of the line numbers in schema_lines containing
+    # pragmas that will be part of the build.
+    classes_not_found = set(class_names) - set(found_classes)
+    if classes_not_found:
+        raise ValueError(
+            _format("Classes {0!A} not in the CIM schema pragma file {1!A}",
+                    ', '.join(classes_not_found), schema_pragma_file))
+
+    # Sort the list so the result is in the same order as the pragmas
+    # in the cim_schema pragma file.
+    lines_list.sort()
+
+    # Create list with line from pragma file for each entry in lines_list
+    output_lines = [schema_lines[line_number] for line_number in lines_list]
+
+    output_lines.insert(0, '#pragma locale ("en_US")\n')
+
+    return ''.join(output_lines)
+
+
 class DMTFCIMSchema(object):
     """
     :class:`DMTFCIMSchema` represents a DMTF CIM Schema downloaded from the
     DMTF web site.
 
+    *New in pywbem 0.13 as experimental and finalized in 1.2.*
+
     This class manages the download of the DMTF schema zip file and extraction
     of MOF files of DMTF CIM schema releases into a directory that can be used
-    by :class:`~pywbem_mock.FakedWBEMConnection` to compile a mock repository
-    with class and qualifier declarations.
+    by :meth:`~pywbem_mock.FakedWBEMConnection.compile_schema_classes` to
+    compile into a CIM repository with class and qualifier declarations from the
+    schema.
     """
     def __init__(self, schema_version, schema_root_dir, use_experimental=False,
                  verbose=False):
@@ -113,7 +213,7 @@ class DMTFCIMSchema(object):
 
         Parameters:
 
-          schema_version  (tuple of 3 integers (m, n, u):
+          schema_version (`tuple of m, n, u`):
             Represents the DMTF CIM schema version m.n.u where:
 
             * m is the DMTF CIM schema major version
@@ -126,12 +226,12 @@ class DMTFCIMSchema(object):
           schema_root_dir (:term:`string`):
             Path name of the schema root directory into which the DMTF CIM
             schema zip file is downloaded and in which a subdirectory for the
-            extracted MOF files is created
+            extracted MOF files for the schema version defined is created
 
             Multiple DMTF CIM schemas may be maintained in the same
             `schema_root_dir` simultaneously because the MOF for each schema is
             extracted into a subdirectory identified by the schema version
-            information.
+            information See :attr:`schema_mof_dir`.
 
           use_experimental (:class:`py:bool`):
             If `True`, the experimental version of the defined DMTF schema is
@@ -153,7 +253,7 @@ class DMTFCIMSchema(object):
         """
         if not isinstance(schema_version, tuple):
             raise TypeError(
-                _format("schema_version {v!A} must be tuple",
+                _format("Schema_version {v!A} must be tuple",
                         v=schema_version))
         if len(schema_version) != 3:
             raise ValueError(
@@ -182,8 +282,8 @@ class DMTFCIMSchema(object):
         schema_mof_bld_name = cim_schema_version + '.mof'
 
         self._schema_zip_file = os.path.join(self._schema_root_dir, mof_zip_bn)
-        self._schema_mof_file = os.path.join(self.schema_mof_dir,
-                                             schema_mof_bld_name)
+        self._schema_pragma_file = os.path.join(self.schema_mof_dir,
+                                                schema_mof_bld_name)
 
         self.verbose = verbose
 
@@ -196,17 +296,20 @@ class DMTFCIMSchema(object):
     @property
     def schema_version(self):
         """
-        :func:`py:tuple`: of 3 integers with major version, minor version, and
-        update version of the DMTF Schema installed. Ex (2, 49, 0) defines DMTF
-        schema version 2.49.0.
+        :class:`py:tuple`: The DMTF CIM schema version as a tuple of 3 integers
+        with major version, minor version, and update version.
+
+        Example: (2, 49, 0) defines DMTF CIM schema version 2.49.0.
         """
         return self._schema_version
 
     @property
     def schema_version_str(self):
         """
-        :term:`string`: with the DMTF CIM schema version in the form
-        <major version>.<minor version>.<update version>.
+        :term:`string`: The DMTF CIM schema version as a string in the form
+        ``major version>.<minor version>.<update version>``.
+
+        Example: "2.49.0" defines DMTF CIM schema version 2.49.0.
         """
         return '{0}.{1}.{2}'.format(*self.schema_version)
 
@@ -214,8 +317,8 @@ class DMTFCIMSchema(object):
     def schema_root_dir(self):
         """
         :term:`string`: Path name of the directory in which the DMTF CIM
-        Schema zip file is downloaded. the MOF files are extracted into a
-        subdirectory, `schema_mof_dir`.
+        schema zip file is downloaded. The MOF files are extracted into the
+        subdirectory indicated by :attr:`schema_mof_dir`.
         """
         return self._schema_root_dir
 
@@ -223,43 +326,47 @@ class DMTFCIMSchema(object):
     def schema_mof_dir(self):
         """
         :term:`string`: Path name of the directory in which the DMTF CIM Schema
-        MOF files are extracted from the downloaded zip file. This property is
-        useful since it can be used as the MOF compiler search path for
-        compiling classes in the DMTF CIM schema.
+        MOF files are extracted from the downloaded zip file. This property can
+        be used as the MOF compiler search path for compiling classes in the
+        DMTF CIM schema. This directory will also contain the
+        schema pragma file.
         """
         return self._schema_mof_dir
 
     @property
-    def schema_mof_file(self):
+    def schema_pragma_file(self):
         """
-        :term:`string`: Path name of the DMTF CIM schema MOF top level file
-        which used to compile the complete CIM schema. This file
-        contains the `#pragama include` statements for all of the classes
-        in the schema and the qualifier declaration `#pragma include` statements
-        to compile `qualifiers.mof` and `qualifiers_optional.mof`.
+        :term:`string`: Path name of the schema pragma file for the DMTF CIM
+        schema. This file contains `#pragama include` statements for all of
+        the classes and qualifier declarations of the schema.
 
-        This filename is of the general form::
+        The classes are represented with one file per class, and the qualifier
+        declarations are in the files `qualifiers.mof` and
+        `qualifiers_optional.mof`.
+
+        The path name of the schema pragma file is of the general form::
 
             <schema_mof_dir>/cim_schema_<schema_version_str>.mof
 
-            ex: schemas/dmtf/moffinal2490/cim_schema_2.49.0.mof
+        Example::
+
+            schemas/dmtf/moffinal2490/cim_schema_2.49.0.mof
         """
-        return self._schema_mof_file
+        return self._schema_pragma_file
 
     @property
     def schema_zip_file(self):
         """
-        :term:`string`: Path name of the DMTF CIM schema zip file
+        :term:`string`: Path name of the DMTF CIM schema zip file after being
         downloaded from the DMTF web site.
-
         """
         return self._schema_zip_file
 
     @property
     def schema_zip_url(self):
         """
-        :term:`string`: Url for the DMTF CIM Schema version that is
-        downloaded.
+        :term:`string`: URL of the DMTF CIM schema zip file that is
+        downloaded from the DMTF web site.
 
         """
         return self._schema_zip_url
@@ -273,11 +380,12 @@ class DMTFCIMSchema(object):
 
         return '{0}(schema_version_str={1}, schema_root_dir={2}, ' \
                'schema_mof_dir={3}, ' \
-               ' schema_mof_file={4})'.format(self.__class__.__name__,
-                                              self.schema_version_str,
-                                              self.schema_root_dir,
-                                              self.schema_mof_dir,
-                                              self.schema_mof_file)
+               'schema_pragma_file={4})'.format(
+                   self.__class__.__name__,
+                   self.schema_version_str,
+                   self.schema_root_dir,
+                   self.schema_mof_dir,
+                   self.schema_pragma_file)
 
     def __repr__(self):
         """
@@ -287,13 +395,13 @@ class DMTFCIMSchema(object):
         """
         return '{0}(schema_version={1}, schema_root_dir={2}, ' \
                'schema_zip_file={3}, schema_mof_dir={4}, ' \
-               'schema_mof_file={5}, schema_zip_url={6})' \
+               'schema_pragma_file={5}, schema_zip_url={6})' \
                .format(self.__class__.__name__,
                        self.schema_version,
                        self.schema_root_dir,
                        self.schema_zip_file,
                        self.schema_mof_dir,
-                       self.schema_mof_file,
+                       self.schema_pragma_file,
                        self.schema_zip_url)
 
     def _setup_dmtf_schema(self):
@@ -353,7 +461,7 @@ class DMTFCIMSchema(object):
                     _format("DMTF Schema archive not found at url {0}: {1}",
                             self.schema_zip_url, ie))
 
-            with open(self.schema_zip_file, 'wb') as fp:
+            with io.open(self.schema_zip_file, 'wb') as fp:
                 for data in ufo:
                     fp.write(data)
 
@@ -362,54 +470,59 @@ class DMTFCIMSchema(object):
                 _format("Creating directory for CIM Schema MOF files: {0}",
                         self.schema_mof_dir))
             os.mkdir(self.schema_mof_dir)
-        if not os.path.isfile(self._schema_mof_file):
+        if not os.path.isfile(self._schema_pragma_file):
             print_verbose(
                 _format("Unpacking CIM Schema archive: {0}",
                         self.schema_zip_file))
 
-            zfp = None
-            try:
-                zfp = ZipFile(self.schema_zip_file, 'r')
-                nlist = zfp.namelist()
-                for file_ in nlist:
-                    dfile = os.path.join(self.schema_mof_dir, file_)
-                    if dfile[-1] == '/':
-                        if not os.path.exists(dfile):
-                            os.mkdir(dfile)
-                    else:
-                        with open(dfile, 'w+b') as dfp:
-                            dfp.write(zfp.read(file_))
-            finally:
-                if zfp:
-                    zfp.close()
+        with ZipFile(self.schema_zip_file, 'r') as zfp:
+            nlist = zfp.namelist()
+            for file_ in nlist:
+                dfile = os.path.join(self.schema_mof_dir, file_)
+                if dfile[-1] == '/':
+                    if not os.path.exists(dfile):
+                        os.mkdir(dfile)
+                else:
+                    with io.open(dfile, 'w+b') as dfp:
+                        dfp.write(zfp.read(file_))
 
-    def build_schema_mof(self, schema_classes):
+    def build_schema_mof(self, class_names):
         """
-        Build a string that includes the ``#include pragma`` statements for the
-        DMTF schema CIM classes defined in `schema_classes` using the DMTF CIM
-        schema defined by this object.
+        Build a string that includes the ``#include pragma`` statement for the
+        CIM schema classes defined in `class_names` using the DMTF CIM
+        schema defined by this instance of the class.
 
-        The class names in this list can be just leaf classes. The pywbem
-        MOF compiler will search for dependent classes.
+        The class names in `class_names` can be just leaf classes within the
+        schema . The pywbem MOF compiler will search for dependent classes
+        including superclasses, and other classes referenced as the classes are
+        compiled.
 
         It builds a compilable MOF string in the form::
 
             #pragma locale ("en_US")
             #pragma include ("System/CIM_ComputerSystem.mof")
+            ...
 
-        with a ``#pragma include`` for each classname in the `schema_classes`
-        list
+        with a ``#pragma include`` for each class name in the `class_names`
+        list.
+
+        The resulting set of #pragma statements is in the same order as the
+        original pragmas.  In some cases that could be important because the
+        cim schema pragma file is ordered to missing dependencies and conflicts
+        between class mof compiles
 
         Parameters:
 
-          schema_classes (:term:`py:list` of :term:`string` or :term:`string`):
+          class_names (:term:`string` or list of :term:`string`):
             These must be class names of classes in the DMTF CIM schema
             represented by this :class:`~pywbem_mock.DMTFCIMSchema` object.
             This parameter can be a string if there is only a single class name
             to be included.
 
             If the returned string is compiled, the MOF compiler will search
-            the directory defined by `schema_mof_dir` for dependent classes.
+            the directory defined by `schema_mof_dir` for classes referenced by
+            each class in this list including super classes, classed defined by
+            reference properties, and classes defined by an embedded-object.
 
         Returns:
             :term:`string`: Valid MOF containing pragma statements for
@@ -417,30 +530,9 @@ class DMTFCIMSchema(object):
 
         Raises:
             ValueError: If any of the classnames in `schema_classes` are not in
-            the DMTF CIM schema installed
+              the DMTF CIM schema installed
         """
-        if isinstance(schema_classes, six.string_types):
-            schema_classes = [schema_classes]
-
-        schema_lines = []
-        with open(self.schema_mof_file, 'r') as f:
-            schema_lines = f.readlines()
-
-        output_lines = ['#pragma locale ("en_US")\n']
-        for cln in schema_classes:
-            test_cln = '/{0}.mof'.format(cln)  # May contain Unicode
-            found = False
-            for line in schema_lines:
-                if line.find(test_cln) != -1:
-                    output_lines.append(line)
-                    found = True
-                    break
-            if not found:
-                raise ValueError(
-                    _format("Class {0!A} not in DMTF CIM schema {1!A}",
-                            cln, self.schema_mof_file))
-
-        return ''.join(output_lines)
+        return build_schema_mof(class_names, self.schema_pragma_file)
 
     def clean(self):
         """
