@@ -45,7 +45,7 @@ from pywbem import CIMError, CIM_ERR_INVALID_PARAMETER, CIM_ERR_NOT_FOUND, \
 
 from pywbem._utils import _format
 
-from pywbem._nocasedict import NocaseDict
+__all__ = []
 
 
 class ResolverMixin(object):  # pylint: disable=too-few-public-methods
@@ -55,7 +55,7 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
     in the repository.
 
     As classes are passed to the create_class method or through the compiler
-    into the mock repository they are incomplete (they define the properties
+    into the CIM repository they are incomplete (they define the properties
     and methods of the defined class as well as a possible superclass
     definition, and qualifiers (ex. override) that define relationships with
     superclasses). When a user requests or other methods like getinstance uses
@@ -76,23 +76,9 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
        attributes such as class_origin, and propagated properties.
 
     """
-    @staticmethod
-    def _test_qualifier_decl(qualifier, qualifier_repo, namespace):
-        """
-        Test that qualifier is in repo and valid.
-        For for conn_lite, ignore this test
-        """
-        if qualifier_repo is None:
-            return
-        if qualifier.name not in qualifier_repo:
-            raise CIMError(
-                CIM_ERR_INVALID_PARAMETER,
-                _format("Qualifier declaration {0!A} required by CreateClass "
-                        "not found in namespace {1!A}.",
-                        qualifier.name, namespace))
 
     @staticmethod
-    def _validate_qualifiers(qualifier_list, qual_repo, new_class, scope):
+    def _validate_qualifiers(qualifiers, qualifier_store, new_class, scope):
         """
         Validate a list of qualifiers against the Qualifier decl in the
         repository.
@@ -102,38 +88,57 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
         3. Whether the qualifier is valid for the given scope.
         4. Whether the qualifier can be overridden.
         5. Whether the qualifier should be propagated to the subclass.
+
+        Parameters:
+
+          qualifiers: (:class:`py:dict` or `NocaseDict`_):
+             Qualifiers to validate
+
+          qualifier_store ():
+
+          new_class (:class:`~pywbem.CIMClass`):
+            The class being validated
+
+        scope (:term:`string`):
+            The scope defined for the object containing the qualifiers.
+            May be 'CLASS', 'PROPERTY', 'METHOD', 'PARAMETER'
+
         """
-        for qname, qvalue in qualifier_list.items():
-            if qname not in qual_repo:
+        for qname, qual in qualifiers.items():
+            if not qualifier_store.object_exists(qname):
                 raise CIMError(
                     CIM_ERR_INVALID_PARAMETER,
                     _format("Qualifier {0!A} used in new class {1!A} "
                             "has no qualifier declaration in repository.",
                             qname, new_class.classname))
-            q_decl = qual_repo[qname]
-            if qvalue.type != q_decl.type:
+            q_decl = qualifier_store.get(qname)
+            if qual.type != q_decl.type:
                 raise CIMError(
                     CIM_ERR_INVALID_PARAMETER,
                     _format("Qualifier {0!A} used in new class {1!A} has "
                             "invalid type {2!A} (Qualifier declaration type: "
                             "{3!A}).",
-                            qname, new_class.classname, qvalue.type,
+                            qname, new_class.classname, qual.type,
                             q_decl.type))
-            if scope not in q_decl.scopes and 'ANY' not in q_decl.scopes:
+
+            # Test for valid scope for this object type ot qualdecl ANY
+            if not q_decl.scopes[scope] and not q_decl.scopes['ANY']:
                 raise CIMError(
                     CIM_ERR_INVALID_PARAMETER,
                     _format("Qualifier {0!A} in new class {1!A} is used in "
-                            "invalid scope {2!A} (Qualifier declaration "
-                            "scopes: {3})",
-                            qname, new_class.classname, scope, q_decl.scopes))
+                            "invalid scope: {2!A}. (QualifierDeclaration "
+                            "scopes: ({3})",
+                            qname, new_class.classname, scope,
+                            ", ".join(
+                                [s for s, v in q_decl.scopes.items() if v])))
 
     @staticmethod
-    def _init_qualifier(qualifier, qual_repo):
+    def _init_qualifier(qualifier, qualifier_store):
         """
         Initialize the flavors of a qualifier from the qualifier repo and
         initialize propagated.
         """
-        qual_dict_entry = qual_repo[qualifier.name]
+        qual_dict_entry = qualifier_store.get(qualifier.name)
         qualifier.propagated = False
         if qualifier.tosubclass is None:
             if qual_dict_entry.tosubclass is None:
@@ -148,77 +153,56 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
         if qualifier.translatable is None:
             qualifier.translatable = qual_dict_entry.translatable
 
-    @staticmethod
-    def _init_qualifier_decl(qualifier_decl, qual_repo):
-        """
-        Initialize the flavors of a qualifier declaration if they are not
-        already set.
-        """
-        assert qualifier_decl.name not in qual_repo
-        if qualifier_decl.tosubclass is None:
-            qualifier_decl.tosubclass = True
-        if qualifier_decl.overridable is None:
-            qualifier_decl.overridable = True
-        if qualifier_decl.translatable is None:
-            qualifier_decl.translatable = False
-
     def _resolve_objects(self, new_objects, superclass_objects, new_class,
-                         superclass, classrepo, qualifier_repo, type_str,
+                         superclass, classrepo, qualifier_store, type_str,
                          verbose=None):
         """
         Resolve a dictionary of objects where the objects can be CIMProperty,
         CIMMethod, or CIMParameter.  This method resolves each of the objects
         in the dictionary, using the superclass if it is defined.
         """
-        if new_objects:   # TODO Future REMOVE. This is test code
-            assert isinstance(new_objects, (dict, NocaseDict))
-            keys = new_objects.keys()
-            assert isinstance(new_objects[keys[0]], (CIMMethod, CIMProperty,
-                                                     CIMParameter))
 
         if not superclass:
             for new_obj in six.itervalues(new_objects):
                 self._set_new_object(new_obj, None, new_class, None,
-                                     qualifier_repo,
+                                     qualifier_store,
                                      False, type_str)
             return
 
-        # process objects if superclass exists
-        for oname, new_obj in six.iteritems(new_objects):
-            if oname not in superclass_objects:
+        # Process objects if superclass exists
+        for obj_name, new_obj in six.iteritems(new_objects):
+            # If obj_name not in superclass, set into new class
+            if obj_name not in superclass_objects:
                 self._set_new_object(new_obj, None, new_class,
-                                     superclass, qualifier_repo,
+                                     superclass, qualifier_store,
                                      False, type_str)
                 continue
 
-            # oname in superclass_objects
-            # TODO: We may have object naming because of override.
-            if 'Override' not in new_objects[oname].qualifiers:
-                if not isinstance(new_objects[oname], CIMParameter):
+            # Error if obj_name in superclass_objects and not override,
+            # except parameters which have no override concept.
+            if 'Override' not in new_objects[obj_name].qualifiers:
+                if not isinstance(new_objects[obj_name], CIMParameter):
                     raise CIMError(
                         CIM_ERR_INVALID_PARAMETER,
                         _format("{0} {1!A} in {2!A} duplicates {0} in "
                                 "{3!A} without override.",
-                                type_str, oname, new_class.classname,
+                                type_str, obj_name, new_class.classname,
                                 superclass.classname))
-
-                # TODO need to finish this.  For now just let
-                # parameter slide. Keep the new one.
                 continue
 
             # process object override
             # get override name
-            override_name = new_objects[oname].qualifiers["override"].value
+            override_name = new_objects[obj_name].qualifiers["override"].value
             if isinstance(new_obj, (CIMParameter, CIMProperty)):
                 if new_obj.type == 'reference':
-                    if override_name != oname:
+                    if override_name != obj_name:
                         raise CIMError(
                             CIM_ERR_INVALID_PARAMETER,
                             _format("Invalid new_class reference "
                                     "{0} {1!A}. in class {2!A}"
                                     "Override must not change {0} "
                                     "name but override name is {3!A}",
-                                    type_str, oname, superclass.classname,
+                                    type_str, obj_name, superclass.classname,
                                     override_name))
             try:
                 super_obj = superclass_objects[override_name]
@@ -227,8 +211,8 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
                     CIM_ERR_INVALID_PARAMETER,
                     _format("Invalid new_class override  {0} {1!A}. in class "
                             "{2!A}. Override name {3!A}} not found in  {3!A}.",
-                            type_str, oname, new_class.classname, override_name,
-                            superclass.classname))
+                            type_str, obj_name, new_class.classname,
+                            override_name, superclass.classname))
 
             # Test if new object characteristics consistent with
             # requirements for that object type
@@ -242,7 +226,7 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
                         _format("Invalid new_class property {0!A}. "
                                 "Does not match overridden property "
                                 "{1!A} in class {2!A}",
-                                oname, super_obj.name,
+                                obj_name, super_obj.name,
                                 superclass.classname))
             elif isinstance(super_obj, CIMMethod):
                 if super_obj.return_type != new_obj.return_type:
@@ -250,7 +234,7 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
                         CIM_ERR_INVALID_PARAMETER,
                         _format("Invalid new_class method {0!A}. "
                                 "Mismatch method  return typein "
-                                "class {1!A}.", oname,
+                                "class {1!A}.", obj_name,
                                 superclass.classname))
             elif isinstance(super_obj, CIMParameter):
                 if super_obj.type != new_obj.type or \
@@ -264,14 +248,15 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
                                 "{0!A} param {1!A}. "
                                 "Does not match signature of "
                                 "overridden method parameters "
-                                "in class {2!A}.", mname, oname,
+                                "in class {2!A}.", mname, obj_name,
                                 superclass.classname))
             else:
+                # This is our programming error
                 assert True, "Invalid Type {0}" .format(type(super_obj))
 
             self._set_new_object(new_obj, super_obj,
                                  new_class,
-                                 superclass, qualifier_repo,
+                                 superclass, qualifier_store,
                                  True, type_str)
 
             # if type is method, resolve the parameters.
@@ -280,24 +265,24 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
                     new_obj.parameters,
                     superclass_objects[new_obj.name].parameters,
                     new_class,
-                    superclass, classrepo, qualifier_repo,
+                    superclass, classrepo, qualifier_store,
                     "Parameter", verbose=verbose)
 
         # Copy objects from from superclass that are not in new_class
         # Placed after loop with items in new_object so they are not part
         # of that loop.
-        for oname, ovalue in six.iteritems(superclass_objects):
-            if oname not in new_objects:
-                new_value = ovalue.copy()
-                new_value.propagated = True
-                assert ovalue.class_origin
-                new_value.class_origin = ovalue.class_origin
-                for qualifier in new_value.qualifiers.values():
+        for obj_name, obj in six.iteritems(superclass_objects):
+            if obj_name not in new_objects:
+                new_obj = obj.copy()
+                new_obj.propagated = True
+                assert obj.class_origin
+                new_obj.class_origin = obj.class_origin
+                for qualifier in new_obj.qualifiers.values():
                     qualifier.propagated = True
-                new_objects[oname] = new_value
+                new_objects[obj_name] = new_obj
 
     def _set_new_object(self, new_obj, inherited_obj, new_class, superclass,
-                        qualifier_repo, propagated, type_str):
+                        qualifier_store, propagated, type_str):
         """
         Set the object attributes for a single object and resolve the
         qualifiers. This sets attributes for Properties, Methods, and
@@ -324,63 +309,51 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
                                  new_class,
                                  superclass,
                                  new_obj.name, type_str,
-                                 qualifier_repo,
+                                 qualifier_store,
                                  propagate=propagated)
 
     def _resolve_qualifiers(self, new_quals, inherited_quals, new_class,
-                            super_class, obj_name, obj_type, qualifier_repo,
-                            propagate=False, verbose=False):
+                            super_class, obj_name, obj_type, qualifier_store,
+                            propagate=False):
         """
         Process the override of qualifiers from the inherited_quals dictionary
         to the new_quals dict following the override rules in DSP0004.
         """
         superclassname = super_class.classname if super_class else None
 
-        # TODO Diagnostic we will keep until really sure of this code
-        if verbose:
-            print("\nRESOLVE sc_name=%s nc_name=%s, obj_name=%s obj_type=%s "
-                  "  propagate=%s" %
-                  (superclassname, new_class.classname, obj_name, obj_type,
-                   propagate))
-            print('\nNEW QUAL')
-            for q, qv in new_quals.items():
-                print('  %s: %r' % (q, qv))
-            print('INHERITEDQ:')
-            if inherited_quals:
-                for q, qv in inherited_quals.items():
-                    print('  %s: %r' % (q, qv))
-
         # If propagate flag not set, initialize the qualfiers
         # by setting flavor defaults and propagated False
         if not propagate:
-            for qname, qvalue in new_quals.items():
-                self._init_qualifier(qvalue, qualifier_repo)
+            for qname, qual in new_quals.items():
+                self._init_qualifier(qual, qualifier_store)
             return
 
         # resolve qualifiers not in inherited object
-        for qname, qvalue in new_quals.items():
+        for qname, qual in new_quals.items():
             if not inherited_quals or qname not in inherited_quals:
-                self._init_qualifier(qvalue, qualifier_repo)
+                self._init_qualifier(qual, qualifier_store)
 
         # resolve qualifiers from inherited object
-        for inh_qname, inh_qvalue in inherited_quals.items():
-            if inh_qvalue.tosubclass:
-                if inh_qvalue.overridable:
-                    # if not in new quals, copy it to the new quals, else ignore
+        for inh_qname, inh_qual in inherited_quals.items():
+            if inh_qual.tosubclass:
+                if inh_qual.overridable:
+                    # if not in new quals, copy to new quals, else ignore
                     if inh_qname not in new_quals:
                         new_quals[inh_qname] = inherited_quals[inh_qname].copy()
                         new_quals[inh_qname].propagated = True
                     else:
                         new_quals[inh_qname].propagated = False
                         self._init_qualifier(new_quals[inh_qname],
-                                             qualifier_repo)
+                                             qualifier_store)
 
                 else:  # not overridable
                     if inh_qname in new_quals:
-                        # allow for same qualifier def in subclass
-                        # TODO should more than value match here.??
+                        # Allow for same qualifier definition in subclass
                         if new_quals[inh_qname].value != \
-                                inherited_quals[inh_qname].value:
+                                inherited_quals[inh_qname].value \
+                                or \
+                                new_quals[inh_qname].type != \
+                                inherited_quals[inh_qname].type:
                             raise CIMError(
                                 CIM_ERR_INVALID_PARAMETER,
                                 _format("Invalid new_class {0!A}:{1!A} "
@@ -388,8 +361,7 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
                                         "in class {3!A}. Not overridable ",
                                         obj_type, obj_name, inh_qname,
                                         new_class.classname))
-                        else:
-                            new_quals[inh_qname].propagated = True
+                        new_quals[inh_qname].propagated = True
 
                     else:  # not in new class, add it
                         new_quals[inh_qname] = inherited_quals[inh_qname].copy()
@@ -397,7 +369,7 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
 
             else:  # not tosubclass, i.e. restricted.
                 if inh_qname in new_quals:
-                    if inh_qvalue.overridable or inh_qvalue.overridable is None:
+                    if inh_qual.overridable or inh_qual.overridable is None:
                         new_quals[inh_qname].propagated = True
 
                     else:
@@ -407,7 +379,7 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
                                     "{1!A} . Restricted in super class {2!A}",
                                     obj_name, inh_qname, superclassname))
 
-    def _resolve_class(self, new_class, namespace, qualifier_repo,
+    def _resolve_class(self, new_class, namespace, qualifier_store,
                        verbose=None):
         """
         Resolve the class defined by new_class by:
@@ -427,11 +399,10 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
 
         if new_class.superclass:
             try:
-                superclass = self._get_class(new_class.superclass,
-                                             namespace=namespace,
-                                             local_only=False,
-                                             include_qualifiers=True,
-                                             include_classorigin=True)
+                superclass = self.get_class(namespace, new_class.superclass,
+                                            local_only=False,
+                                            include_qualifiers=True,
+                                            include_classorigin=True)
             except CIMError as ce:
                 if ce.status_code == CIM_ERR_NOT_FOUND:
                     raise CIMError(
@@ -440,8 +411,7 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
                                 "in namespace {2!A}.",
                                 new_class.superclass, new_class.classname,
                                 namespace))
-                else:
-                    raise
+                raise
         else:
             superclass = None
 
@@ -451,7 +421,7 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
                 raise CIMError(
                     CIM_ERR_INVALID_PARAMETER,
                     _format("New class {0!A} derived from superclass {1!A} "
-                            "in namespace {3!A} which is not Association "
+                            "in namespace {2!A} which is not Association "
                             "Class .",
                             new_class.classname, new_class.superclass,
                             namespace))
@@ -465,51 +435,54 @@ class ResolverMixin(object):  # pylint: disable=too-few-public-methods
                             "non-association class {1!A}",
                             new_prop.name, new_class.classname))
 
-        objects = list(new_class.properties.values())
-        for meth in new_class.methods.values():
-            objects += list(meth.parameters.values())
-
-        # Validate the attributes of all qualifiers in the new class
-        if qualifier_repo:
-            self._validate_qualifiers(new_class.qualifiers, qualifier_repo,
-                                      new_class, 'CLASS')
-            for pvalue in new_class.properties.values():
-                self._validate_qualifiers(pvalue.qualifiers, qualifier_repo,
-                                          new_class, 'PROPERTY')
-            for mvalue in new_class.methods.values():
-                self._validate_qualifiers(mvalue.qualifiers, qualifier_repo,
+        # Validate the attributes of all qualifiers in the new class and that
+        # their value is True
+        if qualifier_store:
+            if 'ASSOCIATION' in new_class.qualifiers and \
+                    new_class.qualifiers['ASSOCIATION']:
+                class_scope = 'ASSOCIATION'
+            elif 'INDICATION' in new_class.qualifiers and \
+                    new_class.qualifiers['INDICATION']:
+                class_scope = 'INDICATION'
+            else:
+                class_scope = 'CLASS'
+            self._validate_qualifiers(new_class.qualifiers, qualifier_store,
+                                      new_class, class_scope)
+            for prop in new_class.properties.values():
+                prop_scope = 'reference' if prop.type == 'reference' \
+                    else 'PROPERTY'
+                self._validate_qualifiers(prop.qualifiers, qualifier_store,
+                                          new_class, prop_scope)
+            for meth in new_class.methods.values():
+                self._validate_qualifiers(meth.qualifiers, qualifier_store,
                                           new_class, 'METHOD')
-                for pvalue in mvalue.parameters.values():
-                    self._validate_qualifiers(pvalue.qualifiers, qualifier_repo,
+                for parm in meth.parameters.values():
+                    self._validate_qualifiers(parm.qualifiers,
+                                              qualifier_store,
                                               new_class, 'PARAMETER')
 
         # resolve class level qualifiers and attributes
         qualdict = superclass.qualifiers if superclass else {}
-        new_class.classorigin = superclass.classname if superclass \
-            else new_class.classname
-        new_class.propagated = True if superclass else False
         self._resolve_qualifiers(new_class.qualifiers,
                                  qualdict,
                                  new_class,
                                  superclass,
                                  new_class.classname, 'class',
-                                 qualifier_repo,
-                                 propagate=False,
-                                 verbose=verbose)
+                                 qualifier_store,
+                                 propagate=False)
 
-        classrepo = self._get_class_repo(namespace)
+        classrepo = self.cimrepository.get_class_store(namespace)
         # resolve properties in new class
         self._resolve_objects(new_class.properties,
                               superclass.properties if superclass else None,
                               new_class, superclass,
-                              classrepo, qualifier_repo, "Property",
+                              classrepo, qualifier_store, "Property",
                               verbose=verbose)
 
         # resolve methods and parameters in new class
         self._resolve_objects(new_class.methods,
                               superclass.methods if superclass else None,
                               new_class, superclass,
-                              classrepo, qualifier_repo, "Method",
-                              verbose=verbose)
+                              classrepo, qualifier_store, "Method")
 
         return new_class
